%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   DSeg software is currently under active development.We welcome comments
%   and contributions to the documentation and code of DSeg to help  us 
%   improve the implementation.
%
%	Related information can be found in the paper :
%
%   DSeg: A dynamic segmentation program to extract backbone patterns for 
%   filamentous bacteria and hyphae structures, March 2018   
%
%   Hanqing Zhang,Niklas Soderholm, Linda Sandblad, Krister Wiklund,
%   Magnus Andersson*
%   * Corresponding author
%
%  Contact information:  Hanqing Zhang :  hanqing.zhang@umu.se
%                        Magnus Andersson : magnus.andersson@umu.se
% 
%  GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
% 
%  Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>
%  Everyone is permitted to copy and distribute verbatim copies
%  of this license document, but changing it is not allowed.
classdef  DSeg_IMPLEMENTATION <  ...
        CLASS_PREP_INTERFACE_Preprocess &...
        CLASS_DETECT_INTERFACE_Detection &...
        CLASS_TRACK_INTERFACE_DataAnalysis

properties (Access = public)
   Config=[];
end       
    methods
        %% ʼ
        function obj=DSeg_IMPLEMENTATION(~)
            %% Control parameters
            obj.Config=DSeg_initialization();
        end
        %% Preprocessing
        function obj=Preprocess(obj,image)    
            %% Ĭϲ
            % Convert to Gray
            if(~ismatrix(image))
                if(obj.Config.PreProcessConfig.Channel~=0)
                    image=image(:,:,obj.Config.PreProcessConfig.Channel);
                else
                    image=rgb2gray(image);
                end
            end
            % Convert to double 0-1
            if(isa(image(1),'uint8'))
                obj.Image=double(image)/255;
            elseif(isa(image(1),'uint16'))
                obj.Image=double(image)/65535;
            else
                obj.Image=double(image);
            end
            %% ѡ 
            % 1. roi
            if(~isempty(obj.Config.PreProcessConfig.ROI) && obj.Config.PreProcessConfig.ROI_ON==1)
                obj.Image= imcrop(obj.Image,obj.Config.PreProcessConfig.ROI);
            end 
            % 2.Resize
            if(obj.Config.PreProcessConfig.Resize.On==1)
                obj.Image=imresize(obj.Image,obj.Config.PreProcessConfig.Resize.Num);
            end
            
            % 3.ǿȱ仯
            if(obj.Config.PreProcessConfig.InverseIntensity==1)
                obj.Image=1-obj.Image;
            end
            
            % .
            % 4.1 ֵ
            if(obj.Config.PreProcessConfig.Medfilt==1)
                obj.Image= medfilt2(obj.Image,[2*obj.Config.PreProcessConfig.KernelSize+1 2*obj.Config.PreProcessConfig.KernelSize+1],'symmetric');
            end
            % 4.2 ˹
            if(obj.Config.PreProcessConfig.Gaussian.On==1)
                obj.Image=imfilter(obj.Image,obj.Config.PreProcessConfig.Gaussian.Kernel,'symmetric');
            end
            % 4.3 
            if(obj.Config.PreProcessConfig.Aniso==1)
                obj.Image = obj.anisodiff2D(obj.Image,...
                                            obj.Config.PreProcessConfig.Aniso_iteration,...
                                            obj.Config.PreProcessConfig.Aniso_step,...
                                            obj.Config.PreProcessConfig.Aniso_kappa,...
                                            obj.Config.PreProcessConfig.Aniso_mode);
            end
            
            if(obj.Config.PreProcessConfig.Backgroundsubtraction==1)
                switch obj.Config.PreProcessConfig.BackgroundsubtractionMethod
                    case 'Gaussian' 
                        obj.Image=obj.Image-imfilter(obj.Image,obj.Config.PreProcessConfig.Backgroundsub.Gaussian.Kernel,'symmetric');
                    case 'Gaussian Norm'
                        obj.Image=obj.Image./(0.01+imfilter(obj.Image,obj.Config.PreProcessConfig.Backgroundsub.Gaussian.Kernel,'symmetric'));
                    otherwise
                end
                obj.Image(obj.Image<0)=0;
                obj.Image=(obj.Image-min(obj.Image(:)))/(max(obj.Image(:))-min(obj.Image(:))+eps);
            end
            
            % Rescaling
            if(obj.Config.PreProcessConfig.IntensityRescaling==1)
                obj.Image=(obj.Image-min(obj.Image(:)))/(max(obj.Image(:))-min(obj.Image(:))+eps);
            end
            
        end     
        %% Thresholding and detection
        function obj=Detect(obj,Image,Option) 
            obj.Config.DetectionConfig.Selection=Option;
            GrayThresh(obj,Image);
            switch obj.Config.DetectionConfig.Method.Name
                case 'SingleCellDetection'
                    SingleCellDetection(obj,Image);
                    RefineCellShape(obj);
                case 'AutoDetection'
                    AutoDetection(obj);
                otherwise
            end
            obj.Config.PreviousFrame=Image;
        end  
        function obj=ClearDetection(obj)
            for inc=1:length(obj.Detection)
                obj.Config.DetectionConfig.SelectedCellList=[obj.Config.DetectionConfig.SelectedCellList obj.Detection(inc).Id];
            end
            obj.Config.DetectionConfig.SelectedCellList = sort(obj.Config.DetectionConfig.SelectedCellList);
            obj.Detection = obj.Detection(false(1,length(obj.Detection)));
            obj.Config.DetectionConfig.Current.ID=[];
        end    
        function obj=DeleteDetection(obj,data_id)
            if(data_id>0 && data_id<=length(obj.Detection))
                obj.Config.DetectionConfig.SelectedCellList=[obj.Config.DetectionConfig.SelectedCellList obj.Detection(data_id).Id];
                obj.Config.DetectionConfig.SelectedCellList = sort(obj.Config.DetectionConfig.SelectedCellList);
                obj.Detection(data_id)=[];
                if(~isempty(obj.Detection))
                    obj.Config.DetectionConfig.Current.ID=length(obj.Detection);
                else
                    obj.Config.DetectionConfig.Current.ID=[];
                end
            end
        end
        function obj=SetDetection(obj,num,field,value)
               obj.Detection(num) = setfield(obj.Detection(num),field,value); 
        end
        function obj=SetMask(obj,Mask)
           obj.DetectionMask=Mask;
        end
        %% Thresholding
        function obj=GrayThresh(obj,CurrentImage)
            if(obj.Config.DetectionConfig.AdaptiveThresh==1)
                Thresholdmatrix = adaptthresh(CurrentImage,'ForegroundPolarity','bright');
                Thresholdmatrix=Thresholdmatrix+obj.Config.DetectionConfig.AutoThreshOffset;
                obj.DetectionMask = (CurrentImage-Thresholdmatrix)>0;
            else
                if(obj.Config.DetectionConfig.AutoThresh==0)
                    obj.Config.DetectionConfig.EdgeThresh=max(0.01,obj.Config.DetectionConfig.EdgeThresh);
                else
                    obj.Config.DetectionConfig.EdgeThresh=graythresh(CurrentImage)+obj.Config.DetectionConfig.AutoThreshOffset;
                end
                if(obj.Config.DetectionConfig.EdgeThresh>=1 || obj.Config.DetectionConfig.EdgeThresh<=0)
                   obj.Config.DetectionConfig.EdgeThresh=graythresh(CurrentImage); 
                   obj.Config.DetectionConfig.AutoThreshOffset=0;
                   disp('The threshold value must be set within a range of [0 1].');
                end
                obj.DetectionMask = imbinarize(CurrentImage,obj.Config.DetectionConfig.EdgeThresh);
            end
        end      
        %% Selection & Segmentation
        % Method 1
        function obj=SingleCellDetection(obj,CurrentImage) 
            % seedͼֵҵobjectӦmask
            if(~isempty(obj.Config.DetectionConfig.Method.SEED))
                % Object length
                obj_length_range=[obj.Config.DetectionConfig.MinSize obj.Config.DetectionConfig.MaxSize]/obj.Config.PreProcessConfig.Resize.Num;
                
                % ҳʼmask
                if(obj.Config.DetectionConfig.Selection==1)
                    % seedmask
                    Seed_Img=zeros(size(CurrentImage));
                    if(size(obj.Config.DetectionConfig.Method.SEED,2)==1)
                        linearInd = obj.Config.DetectionConfig.Method.SEED;
                        Seed_Img(linearInd)=1;
                    else
                        linearInd = sub2ind(size(CurrentImage), round(obj.Config.DetectionConfig.Method.SEED(:,1)), round(obj.Config.DetectionConfig.Method.SEED(:,2)));
                        Seed_Img(linearInd)=1;
                        obj.Config.DetectionConfig.DistTransMatrix=bwdist(Seed_Img);
                        [BW_Seed_Background,~]=obj.GetBinarySeedPoints(obj.DetectionMask,Seed_Img);
                        Seed_Img=xor(obj.DetectionMask,BW_Seed_Background);
                    end
                else
                    BG_BW_threshold=double(imgaussfilt(double(obj.Config.DetectionConfig.CurrentBackgroundMask))>obj.Config.DetectionConfig.BackgroundThreholdLevel); 
                    dilation_width=max(1,round((obj.Detection(obj.Config.DetectionConfig.Current.ID).Width-1)/2));
                    Seed_Img=bwmorph(obj.Config.DetectionConfig.SkeletonNoBranchMask,'dilate',dilation_width);
                    %Seed_Img=obj.Config.DetectionConfig.SkeletonNoBranchMask;
                    %Seed_Img=or(Seed_Img,obj.Config.DetectionConfig.SkeletonMask);
                end
                % LevelSet
                Seed_length = regionprops(Seed_Img,'MajorAxisLength');
                if(~isempty(Seed_length))
                    CheckZero=Seed_length(1).MajorAxisLength;
                else
                    CheckZero=0;
                end
                if(obj.Config.DetectionConfig.Selection==0 || (CheckZero>=obj_length_range(1) && CheckZero<=obj_length_range(2)))
                    if(obj.Config.DetectionConfig.Selection==1)
                        [Image_LevelSet,EdgeMatrix,~]=FUNC_LocalMinimumVaraiceLevelset(CurrentImage...
                            ,obj.DetectionMask...
                            ,Seed_Img...
                            ,zeros(size(Seed_Img))...
                            ,obj.Config.DetectionConfig.BackgroundBiasWeight...
                            ,obj.Config.DetectionConfig.LEVELSET);
                    else                 
                        [Image_LevelSet,EdgeMatrix,~]=FUNC_LocalMinimumVaraiceLevelset(CurrentImage...
                            ,obj.DetectionMask...
                            ,Seed_Img...
                            ,BG_BW_threshold...
                            ,obj.Config.DetectionConfig.BackgroundBiasWeight...
                            ,obj.Config.DetectionConfig.LEVELSET);  
                        %Image_LevelSet=(double(Image_LevelSet)-BG_BW_threshold)>0;
                    end 
                    % levelset֮Ƿ񺭸seed
                    [~,SeedNum]=obj.GetBinarySeedPoints(Image_LevelSet,Seed_Img);
                    if(SeedNum==0)
                        if(sum(Image_LevelSet(:))>0)
                            Image_LevelSetUpdate=bwmorph(Image_LevelSet,'close');
                        else
                            Image_LevelSetUpdate=bwmorph(Seed_Img,'close');
                        end
                    else
                        Image_LevelSetUpdate=Image_LevelSet; %?
                    end
                    
                    Image_skeleton=bwmorph(Image_LevelSetUpdate,'thin',inf); 
                    % Update background
                    BImage_Background=FUNC_LEVELSET_shapeC(CurrentImage,obj.DetectionMask,EdgeMatrix,Image_LevelSetUpdate,obj.Config.DetectionConfig.BackgroundBiasWeight,obj.Config.DetectionConfig.LEVELSET);
                    [BImage_Background,~]=obj.GetBinarySeedPoints(BImage_Background,Image_skeleton);

                    % !!!  Mask
                    if(obj.Config.DetectionConfig.Selection==1)
                        % Update Object Mask
                        obj.Config.DetectionConfig.PreviousMask = zeros(size(Image_LevelSetUpdate));
                        obj.Config.DetectionConfig.CurrentMask= Image_LevelSetUpdate;
                        % Skeleton Extraction
                        obj.Config.DetectionConfig.PreviousSkeletonMask=zeros(size(Image_LevelSetUpdate));
                        obj.Config.DetectionConfig.SkeletonMask=Image_skeleton;
                        % Update Intensity Mask
                        obj.Config.DetectionConfig.PreviousBackgroundMask=zeros(size(Image_LevelSetUpdate));
                        obj.Config.DetectionConfig.CurrentBackgroundMask=BImage_Background;
                        obj.Config.DetectionConfig.AdaptiveK=obj.Config.DetectionConfig.CurrentBackgroundMaskRatio*ones(size(BImage_Background));
                    else
                        if(isempty(obj.Config.DetectionConfig.PreviousBackgroundMask))
                            obj.Config.DetectionConfig.CurrentBackgroundMask=BImage_Background;
                        end
                        % Update Object Mask
                        obj.Config.DetectionConfig.PreviousMask = obj.Config.DetectionConfig.CurrentMask;
                        obj.Config.DetectionConfig.CurrentMask = Image_LevelSetUpdate;
                        % Skeleton Extraction
                        obj.Config.DetectionConfig.PreviousSkeletonMask=obj.Config.DetectionConfig.SkeletonMask;
                        obj.Config.DetectionConfig.SkeletonMask=Image_skeleton;
                        % Update Intensity Mask
                        obj.Config.DetectionConfig.PreviousBackgroundMask=obj.Config.DetectionConfig.CurrentBackgroundMask;
                        obj.Config.DetectionConfig.CurrentBackgroundMask=...
                            obj.Config.DetectionConfig.AdaptiveK.*...
                            obj.Config.DetectionConfig.CurrentBackgroundMask...
                            +(1-obj.Config.DetectionConfig.AdaptiveK).*...
                            BImage_Background;
                        
                        % Update k 
                        Addmask=and(obj.Config.DetectionConfig.PreviousBackgroundMask,obj.Config.DetectionConfig.CurrentBackgroundMask);
                        Addmask2=and(obj.Config.DetectionConfig.CurrentMask,obj.Config.DetectionConfig.PreviousMask);
                        Submask=and(obj.Config.DetectionConfig.CurrentMask,obj.Config.DetectionConfig.PreviousBackgroundMask);
                        obj.Config.DetectionConfig.AdaptiveK=obj.Config.DetectionConfig.AdaptiveK+...
                                                             obj.Config.DetectionConfig.CurrentBackgroundMaskRatioStep*...
                                                             (double(or(Addmask,Addmask2))-double(Submask));
                        obj.Config.DetectionConfig.AdaptiveK(obj.Config.DetectionConfig.AdaptiveK>=1)=0.99;
                        obj.Config.DetectionConfig.AdaptiveK(obj.Config.DetectionConfig.AdaptiveK<0.6)=0.6;
                    end
                else
                    obj.Config.DetectionConfig.PreviousMask =[];
                    obj.Config.DetectionConfig.CurrentMask=Seed_Img;
                    obj.Config.DetectionConfig.PreviousSkeletonMask=[];
                    obj.Config.DetectionConfig.SkeletonMask=[];
                    obj.Config.DetectionConfig.SkeletonNoBranchMask=[];
                    obj.Config.DetectionConfig.PreviousSkeletonNoBranchMask=[];
                    obj.Config.DetectionConfig.SkeletonNoBranchWeight=[];
                    %obj.DetectionMask=obj.Config.DetectionConfig.CurrentMask;
                end
            else % When not seeded
                % Update Mask
                obj.Config.DetectionConfig.PreviousMask =[];
                obj.Config.DetectionConfig.CurrentMask=[];
                obj.Config.DetectionConfig.PreviousSkeletonMask=[];
                obj.Config.DetectionConfig.SkeletonMask=[];
                obj.Config.DetectionConfig.SkeletonNoBranchMask=[];
                obj.Config.DetectionConfig.PreviousSkeletonNoBranchMask=[];
                obj.Config.DetectionConfig.SkeletonNoBranchWeight=[];
            end 
        end    
        function obj=RefineCellShape(obj)
             % maskǷڻȷ
             if(~isempty(obj.Config.DetectionConfig.SkeletonMask))
              %% Constraints 
                % Object length
                obj_length_range=[obj.Config.DetectionConfig.MinSize obj.Config.DetectionConfig.MaxSize]/obj.Config.PreProcessConfig.Resize.Num;
                % object shape
                SkeletonMask=obj.Config.DetectionConfig.SkeletonMask;
                if(sum(SkeletonMask(:))>3) % 
                    % ±
                    obj.Config.DetectionConfig.ValidMask=1;
                    % Ȩѡ·
                    if(obj.Config.DetectionConfig.Selection==1)
                        if(isempty(obj.Config.DetectionConfig.DistTransMatrix))
                            DistTransWeight=ones(size(obj.Config.DetectionConfig.SkeletonMask));
                        else
                            DistTransWeight=obj.Config.DetectionConfig.NoBranchWeightMagnitude./(1+obj.Config.DetectionConfig.DistTransMatrix);
                        end
                        BW_SingleBranch=FindmaximumlengthBW(obj.Config.DetectionConfig.SkeletonMask,...
                                DistTransWeight);
                    else
                         BW_SingleBranch=FindmaximumlengthBW(obj.Config.DetectionConfig.SkeletonMask,...
                                obj.Config.DetectionConfig.NoBranchWeightMagnitude*(obj.Config.DetectionConfig.SkeletonNoBranchWeight-0.2*obj.Config.DetectionConfig.CurrentBackgroundMask));
                    end
                    BWbranch = regionprops(bwmorph(xor(SkeletonMask,BW_SingleBranch),'clean'),'PixelIdxList');
                    %figure(4);imshow(SkeletonMask);
                    
                else
                    obj.Config.DetectionConfig.ValidMask=0;
                    BW_SingleBranch=obj.Config.DetectionConfig.SkeletonMask;
                    BWbranch = [];
                    DistTransWeight=ones(size(obj.Config.DetectionConfig.SkeletonMask));
                end 
                
                % mask
                if(obj.Config.DetectionConfig.Selection==1)
                    obj.Config.DetectionConfig.PreviousSkeletonNoBranchMask=zeros(size(BW_SingleBranch));
                    obj.Config.DetectionConfig.SkeletonNoBranchMask=BW_SingleBranch;
                    obj.Config.DetectionConfig.SkeletonNoBranchWeight=DistTransWeight.*BW_SingleBranch;
                else
                    obj.Config.DetectionConfig.PreviousSkeletonNoBranchMask=obj.Config.DetectionConfig.SkeletonNoBranchMask;
                    obj.Config.DetectionConfig.SkeletonNoBranchMask=BW_SingleBranch;
                    if(~isempty(obj.Config.DetectionConfig.SkeletonNoBranchWeight))
                        obj.Config.DetectionConfig.SkeletonNoBranchWeight=(1-obj.Config.DetectionConfig.NoBranchUpdateRatio)*obj.Config.DetectionConfig.SkeletonNoBranchWeight+obj.Config.DetectionConfig.NoBranchUpdateRatio*bwmorph(BW_SingleBranch,'dilate',1);
                    else
                        obj.Config.DetectionConfig.SkeletonNoBranchWeight=BW_SingleBranch;
                    end
                end
                
                %
                %Shape=regionprops(obj.Config.DetectionConfig.CurrentMask,'PixelIdxList');
                if(obj.Config.DetectionConfig.ShowSkeletonNoBranchMask==1)
                    BWFeatures = regionprops(BW_SingleBranch,'MajorAxisLength','Centroid','BoundingBox','PixelIdxList','Orientation');
                    obj.Config.DetectionConfig.CurrentMask=bwmorph(BW_SingleBranch,'dilate');
                else
                    BWFeatures = regionprops(obj.Config.DetectionConfig.CurrentMask,'MajorAxisLength','Centroid','BoundingBox','PixelIdxList','Orientation');
                end
                
                %%
                if(obj.Config.DetectionConfig.Add==1)
                    for Blob_i=1:size(BWFeatures,1)
                        if(BWFeatures(Blob_i).MajorAxisLength>=obj_length_range(1) && BWFeatures(Blob_i).MajorAxisLength<obj_length_range(2))
                            % Take Id from list
                            if(~isempty(obj.Config.DetectionConfig.SelectedCellList))
                                inc=length(obj.Detection);
                                idn=obj.Config.DetectionConfig.SelectedCellList(1);
                                obj.Config.DetectionConfig.SelectedCellList(1)=[];
                            
                                CellWidthDist=bwdist(xor(bwmorph(obj.Config.DetectionConfig.CurrentMask,'dilate'),obj.Config.DetectionConfig.CurrentMask));
                                CellWidth=mean(CellWidthDist(BWFeatures(Blob_i).PixelIdxList));
                                
                                NewDetection = struct(...
                                    'Id',idn,...
                                    'ROI', floor(BWFeatures(Blob_i).BoundingBox),...
                                    'Centroid', BWFeatures(Blob_i).Centroid,...
                                    'Width',2*CellWidth+1,...
                                    'Length',2*CellWidth+BWFeatures(Blob_i).MajorAxisLength,...
                                    'Area', (2*CellWidth+1)*BWFeatures(Blob_i).MajorAxisLength,...
                                    'Orientation', BWFeatures(Blob_i).Orientation,...
                                    'Growingspeed', 0,...
                                    'Completed', 0,...
                                    'CurrentFrame', obj.Config.FrameNum,...
                                    'StartFrame',obj.Config.FrameNum,...
                                    'EndFrame',min(obj.Config.FrameNum+obj.Config.ProcessFrames-1,round(obj.Config.AVIobj.Duration*obj.Config.AVIobj.FrameRate)),...
                                    'SeedPoints',obj.Config.DetectionConfig.Method.SEED,...
                                    'PixelIDs',BWFeatures(Blob_i).PixelIdxList,...
                                    'Branch',BWbranch); %'Endpoints',[EP_X;EP_Y]);
                                obj.Detection(inc+1)=NewDetection;
                                obj.Config.DetectionConfig.Current.ID=idn;
                            end
                        end
                    end
                else
                    if(~isempty(obj.Config.DetectionConfig.Current.ID))
                        update_Detection_id=0;
                        for i_f=1:length(obj.Detection)
                            if(obj.Config.DetectionConfig.Current.ID==i_f)
                               update_Detection_id= i_f;break;
                            end
                        end
                         if(size(BWFeatures,1)==1 && update_Detection_id~=0)
                            if(BWFeatures(1).MajorAxisLength>=obj_length_range(1) && BWFeatures(1).MajorAxisLength<obj_length_range(2))
                                CellWidthDist=bwdist(xor(bwmorph(obj.Config.DetectionConfig.CurrentMask,'dilate'),obj.Config.DetectionConfig.CurrentMask));
                                CellWidth=mean(CellWidthDist(BWFeatures(1).PixelIdxList));
                                
                                obj.Detection(update_Detection_id).Width=2*CellWidth+1;
                                obj.Detection(update_Detection_id).Length=2*CellWidth+BWFeatures(1).MajorAxisLength;
                                
                                NewArea=(2*CellWidth+1)*obj.Detection(update_Detection_id).Length;
                                obj.Detection(update_Detection_id).Growingspeed= NewArea-obj.Detection(update_Detection_id).Area;
                                obj.Detection(update_Detection_id).Area= NewArea;
                                obj.Detection(update_Detection_id).PixelIDs=BWFeatures(1).PixelIdxList;
                                obj.Detection(update_Detection_id).Branch=BWbranch;
                                obj.Detection(update_Detection_id).Orientation= BWFeatures(1).Orientation;
                            end
                         end
                    end
                end
             else
                 obj.Config.DetectionConfig.ValidMask=0;
             end
        end
        % Method 2
        function obj=AutoDetection(obj)
            obj_length_range=[obj.Config.DetectionConfig.MinSize obj.Config.DetectionConfig.MaxSize]/obj.Config.PreProcessConfig.Resize.Num;
            [sx,sy,~]=size(obj.DetectionMask);
            BWFeatures = regionprops(obj.DetectionMask,'Area','MajorAxisLength','Centroid','BoundingBox','PixelIdxList');
            obj.Config.DetectionConfig.ValidMask=0;
            for Blob_i=1:size(BWFeatures,1)
                if(BWFeatures(Blob_i).MajorAxisLength>=obj_length_range(1) && BWFeatures(Blob_i).MajorAxisLength<=obj_length_range(2))
                    % Take Id from list
                    if(~isempty(obj.Config.DetectionConfig.SelectedCellList))
                        obj.Config.DetectionConfig.ValidMask=1;
                        inc=length(obj.Detection);
                        idn=obj.Config.DetectionConfig.SelectedCellList(1);
                        obj.Config.DetectionConfig.SelectedCellList(1)=[];
                        Blob_object=zeros(sx,sy);
                        Blob_object(BWFeatures(Blob_i).PixelIdxList)=1;
                        
                        Blob_skeleton=bwmorph(Blob_object,'thin',inf);
                        DistTransWeight=Blob_skeleton;
                        BW_SingleBranch=FindmaximumlengthBW(Blob_skeleton,...
                                        DistTransWeight);
                                    
                        Shape=regionprops(Blob_object,'PixelIdxList');
                        BW_SingleFeatures=regionprops(BW_SingleBranch,'Area','Orientation','PixelIdxList');
                        if(~isempty(BW_SingleFeatures))
                            CClist=BW_SingleFeatures(1).PixelIdxList;
                            centerid=CClist(round(length(CClist)/2));
                            [xi,yi]=ind2sub([sx,sy],centerid);
                        else
                            BW_SingleFeatures=regionprops(Blob_skeleton,'Area','Orientation','PixelIdxList');
                            CClist=BW_SingleFeatures(1).PixelIdxList;
                            centerid=CClist(round(length(CClist)/2));
                            [xi,yi]=ind2sub([sx,sy],centerid);
                        end
                        CClist=Shape(1).PixelIdxList;
                        
                        CellWidthDist=bwdist(xor(bwmorph(Blob_object,'dilate'),Blob_object));
                        CellWidth=max(0,mean(CellWidthDist(BWFeatures(Blob_i).PixelIdxList))-1);
                        
                        NewDetection = struct(...
                            'Id',idn,...
                            'ROI', floor(BWFeatures(Blob_i).BoundingBox),...
                            'Centroid', BWFeatures(Blob_i).Centroid,...
                            'Width',2*CellWidth+1,...
                            'Length',2*CellWidth+BW_SingleFeatures(1).Area,...
                            'Area', BWFeatures(Blob_i).Area,...
                            'Orientation', BW_SingleFeatures(1).Orientation,...
                            'Growingspeed', 0,...
                            'Completed', 0,...
                            'CurrentFrame', obj.Config.FrameNum,...
                            'StartFrame',obj.Config.FrameNum,...
                            'EndFrame',min(obj.Config.FrameNum+obj.Config.ProcessFrames-1,round(obj.Config.AVIobj.Duration*obj.Config.AVIobj.FrameRate)),...
                            'SeedPoints',[xi,yi],...
                            'PixelIDs',CClist,...
                            'Branch',1); %'Endpoints',[EP_X;EP_Y]);
                        obj.Detection(inc+1)=NewDetection;
                        obj.Config.DetectionConfig.Current.ID=idn;
                    else
                        break;
                    end
                end
            end
            if(obj.Config.DetectionConfig.ValidMask==1)
                obj.Config.DetectionConfig.PreviousSkeletonNoBranchMask=zeros(sx,sy);
                obj.Config.DetectionConfig.SkeletonNoBranchMask=BW_SingleBranch;
                obj.Config.DetectionConfig.SkeletonNoBranchWeight=DistTransWeight.*BW_SingleBranch;
                % Update Object Mask
                obj.Config.DetectionConfig.PreviousMask = zeros(sx,sy);
                obj.Config.DetectionConfig.CurrentMask= Blob_object;
                % Skeleton Extraction
                obj.Config.DetectionConfig.PreviousSkeletonMask=zeros(sx,sy);
                obj.Config.DetectionConfig.SkeletonMask=Blob_skeleton;
                % Update Intensity Mask
                obj.Config.DetectionConfig.PreviousBackgroundMask=zeros(sx,sy);
                BImage_Background=(obj.DetectionMask-Blob_object)>0;
                obj.Config.DetectionConfig.CurrentBackgroundMask=BImage_Background;
                obj.Config.DetectionConfig.AdaptiveK=obj.Config.DetectionConfig.CurrentBackgroundMaskRatio*ones(size(BImage_Background));
            else
                obj.Config.DetectionConfig.PreviousMask =[];
                obj.Config.DetectionConfig.CurrentMask=[];
                obj.Config.DetectionConfig.PreviousSkeletonMask=[];
                obj.Config.DetectionConfig.SkeletonMask=[];
                obj.Config.DetectionConfig.SkeletonNoBranchMask=[];
                obj.Config.DetectionConfig.PreviousSkeletonNoBranchMask=[];
                obj.Config.DetectionConfig.SkeletonNoBranchWeight=[];
            end
        end
       %% shape analysis (branch num==1)
        function obj=Analyze(obj,FrameNum)
             Conv_factor=obj.Config.System.PixelSpacing;
             Resize_Num=obj.Config.PreProcessConfig.Resize.Num;
             if(Resize_Num==0)
                Resize_Num=1; 
             else
                 Conv_factor=Conv_factor/Resize_Num;
             end
             if(~isempty(obj.Config.DetectionConfig.SkeletonNoBranchMask))
                        ALLResults=CellLengthAnalysis(obj.Config.DetectionConfig.SkeletonNoBranchMask,obj.Config.AnalysisConfig,Resize_Num);
                        if(ALLResults.Est_PersistanceLength_Cos(1)<0) % avoid errors in short segment
                            ALLResults.Est_PersistanceLength_Cos(1)=ALLResults.E2ELength_parameter;
                        end
                        if(ALLResults.Area~=0)
                        obj.Results.Name={'Object Id' 'Frame Number' 'Width' 'Length (end to end)' 'Area' 'Oerientation' 'Growing Speed' 'Maximum Angle' 'Backbone length' 'Persistence length <Cos>' 'Persistence length <R2>'};
                        obj.Results.Data=...
                            [obj.Detection(obj.Config.DetectionConfig.Current.ID).Id ...
                            FrameNum ...
                            obj.Detection(obj.Config.DetectionConfig.Current.ID).Width*Conv_factor ...
                            ALLResults.E2ELength_parameter ...
                            obj.Detection(obj.Config.DetectionConfig.Current.ID).Area*Conv_factor*Conv_factor ...
                            obj.Detection(obj.Config.DetectionConfig.Current.ID).Orientation ...
                            obj.Detection(obj.Config.DetectionConfig.Current.ID).Growingspeed*Conv_factor*Conv_factor ...
                            ALLResults.maximum_angle ...
                            ALLResults.CurveLength_parameter ...
                            ALLResults.Est_PersistanceLength_Cos(1)... %  unit micrometer
                            ALLResults.Est_PersistanceLength_R2(1)];  %  unit micrometer
                        else
                             obj.Results.Name={'Object Id' 'Frame Number' 'Width' 'Length (fitting)' 'Area' 'Oerientation' 'Growing Speed' 'Maximum Angle' 'Backbone length' 'Persistence length <Cos>' 'Persistence length <R2>'};
                        obj.Results.Data=...
                            [obj.Detection(obj.Config.DetectionConfig.Current.ID).Id ...
                            FrameNum ...
                            obj.Detection(obj.Config.DetectionConfig.Current.ID).Width*Conv_factor ...
                            obj.Detection(obj.Config.DetectionConfig.Current.ID).Length*Conv_factor ...
                            obj.Detection(obj.Config.DetectionConfig.Current.ID).Area*Conv_factor*Conv_factor ...
                            obj.Detection(obj.Config.DetectionConfig.Current.ID).Orientation ...
                            obj.Detection(obj.Config.DetectionConfig.Current.ID).Growingspeed*Conv_factor*Conv_factor ...
                            ALLResults.maximum_angle ...
                            ALLResults.CurveLength_parameter ...
                            ALLResults.Est_PersistanceLength_Cos(1)... %  unit micrometer
                            ALLResults.Est_PersistanceLength_R2(1)];  %  unit micrometer
                        end
             end
        end
       %% Tracking (unused)
    end
    methods(Static)
        % Object selection
        function [BW_Seed,SeedNum]=GetBinarySeedPoints(BWIm,BWCheck)
                    linearInd=find(BWCheck==1);
                    SeedNum=0;
                    CC = bwconncomp(BWIm,8);
                    BW_Seed=BWIm;
                    for ObjCandidate_i=1:length(linearInd)
                        for Blob_i=1:length(CC.PixelIdxList)
                            if(sum(CC.PixelIdxList{Blob_i}==linearInd(ObjCandidate_i))>0)
                                BW_Seed(CC.PixelIdxList{Blob_i}) = 0;
                                CC.PixelIdxList{Blob_i}=[];
                                CC.PixelIdxList=CC.PixelIdxList(~cellfun(@isempty,CC.PixelIdxList));
                                SeedNum=SeedNum+1;
                                break;
                            end
                        end
                    end
        end
        % Template
        function ROI=SelectROI(Image,axes1_handle)
            [FRAME_WIDTH, FRAME_HIGHT]=size(Image);
            h = imrect(axes1_handle, [1   1  FRAME_WIDTH-1 FRAME_HIGHT-1]);
            setResizable(h,1)
            ROI=[1   1  FRAME_WIDTH-1 FRAME_HIGHT-1];
            try
                position = wait(h); % Double click
                if(isempty(position))
                   return; 
                end
                if(position(1)<1 || position(1)>=FRAME_WIDTH)
                    position(3)=position(3)-abs(position(1));
                    position(1)=1;
                end
                if(position(2)<1 || position(2)>=FRAME_HIGHT)
                    position(4)=position(4)-abs(position(2));
                    position(2)=1;
                end
                if(position(1)+position(3)>FRAME_WIDTH || position(3)<=0)
                    position(3)=FRAME_WIDTH-position(1);
                end
                if(position(2)+position(4)>FRAME_HIGHT || position(4)<=0)
                    position(4)=FRAME_HIGHT-position(2);
                end
                ROI=uint32(position);
            catch
               disp('The selection of region-of-interest is cancled.') ;
            end
        end
        function vidtemplate=cut_template(videoFrame,center,d_size)
            
            d_size=int32(d_size);
            center=round(center);
            
            if(center(1)-d_size<=0)
                roi_1=1;
                temp_1=d_size-center(1)+2;
            else
                roi_1=center(1)-d_size;
                temp_1=1;
            end
            if(center(2)-d_size<=0)
                roi_2=1;
                temp_2=d_size-center(2)+2;
            else
                roi_2=center(2)-d_size;
                temp_2=1;
            end
            if(center(1)+d_size>size(videoFrame,2))
                roi_3=size(videoFrame,2);
                temp_3=2*d_size+1-(center(1)+d_size-size(videoFrame,2));
            else
                roi_3=center(1)+d_size;
                temp_3=2*d_size+1;
            end
            if(center(2)+d_size>size(videoFrame,1))
                roi_4=size(videoFrame,1);
                temp_4=2*d_size+1-(center(2)+d_size-size(videoFrame,1));
            else
                roi_4=center(2)+d_size;
                temp_4=2*d_size+1;
            end
            
            vidtemplate=zeros(2*d_size+1,2*d_size+1);
            vidtemplate(temp_2:temp_4,temp_1:temp_3)=videoFrame(roi_2:roi_4,roi_1:roi_3);
        end
        % Anisotropic diffusion
        function diff_im = anisodiff2D(im, num_iter, delta_t, kappa, option)
            %ANISODIFF2D Conventional anisotropic diffusion
            %   DIFF_IM = ANISODIFF2D(IM, NUM_ITER, DELTA_T, KAPPA, OPTION) perfoms
            %   conventional anisotropic diffusion (Perona & Malik) upon a gray scale
            %   image. A 2D network structure of 8 neighboring nodes is considered for
            %   diffusion conduction.
            %
            %       ARGUMENT DESCRIPTION:
            %               IM       - gray scale image (MxN).
            %               NUM_ITER - number of iterations.
            %               DELTA_T  - integration constant (0 <= delta_t <= 1/7).
            %                          Usually, due to numerical stability this
            %                          parameter is set to its maximum value.
            %               KAPPA    - gradient modulus threshold that controls the conduction.
            %               OPTION   - conduction coefficient functions proposed by Perona & Malik:
            %                          1 - c(x,y,t) = exp(-(nablaI/kappa).^2),
            %                              privileges high-contrast edges over low-contrast ones.
            %                          2 - c(x,y,t) = 1./(1 + (nablaI/kappa).^2),
            %                              privileges wide regions over smaller ones.
            %
            %       OUTPUT DESCRIPTION:
            %                DIFF_IM - (diffused) image with the largest scale-space parameter.
            %
            %   Example
            %   -------------
            %   s = phantom(512) + randn(512);
            %   num_iter = 15;
            %   delta_t = 1/7;
            %   kappa = 30;
            %   option = 2;
            %   ad = anisodiff2D(s,num_iter,delta_t,kappa,option);
            %   figure, subplot 121, imshow(s,[]), subplot 122, imshow(ad,[])
            %
            % See also anisodiff1D, anisodiff3D.
            
            % References:
            %   P. Perona and J. Malik.
            %   Scale-Space and Edge Detection Using Anisotropic Diffusion.
            %   IEEE Transactions on Pattern Analysis and Machine Intelligence,
            %   12(7):629-639, July 1990.
            %
            %   G. Grieg, O. Kubler, R. Kikinis, and F. A. Jolesz.
            %   Nonlinear Anisotropic Filtering of MRI Data.
            %   IEEE Transactions on Medical Imaging,
            %   11(2):221-232, June 1992.
            %
            %   MATLAB implementation based on Peter Kovesi's anisodiff(.):
            %   P. D. Kovesi. MATLAB and Octave Functions for Computer Vision and Image Processing.
            %   School of Computer Science & Software Engineering,
            %   The University of Western Australia. Available from:
            %   <http://www.csse.uwa.edu.au/~pk/research/matlabfns/>.
            %
            % Credits:
            % Daniel Simoes Lopes
            % ICIST
            % Instituto Superior Tecnico - Universidade Tecnica de Lisboa
            % danlopes (at) civil ist utl pt
            % http://www.civil.ist.utl.pt/~danlopes
            %
            % May 2007 original version.
            
            % Convert input image to double.
            im = double(im);
            
            % PDE (partial differential equation) initial condition.
            diff_im = im;
            
            % Center pixel distances.
            dx = 1;
            dy = 1;
            dd = sqrt(2);
            
            % 2D convolution masks - finite differences.
            hN = [0 1 0; 0 -1 0; 0 0 0];
            hS = [0 0 0; 0 -1 0; 0 1 0];
            hE = [0 0 0; 0 -1 1; 0 0 0];
            hW = [0 0 0; 1 -1 0; 0 0 0];
            hNE = [0 0 1; 0 -1 0; 0 0 0];
            hSE = [0 0 0; 0 -1 0; 0 0 1];
            hSW = [0 0 0; 0 -1 0; 1 0 0];
            hNW = [1 0 0; 0 -1 0; 0 0 0];

            for t = 1:num_iter
                nablaN = imfilter(diff_im,hN,'conv','symmetric');
                nablaS = imfilter(diff_im,hS,'conv','symmetric');
                nablaW = imfilter(diff_im,hW,'conv','symmetric');
                nablaE = imfilter(diff_im,hE,'conv','symmetric');
                nablaNE = imfilter(diff_im,hNE,'conv','symmetric');
                nablaSE = imfilter(diff_im,hSE,'conv','symmetric');
                nablaSW = imfilter(diff_im,hSW,'conv','symmetric');
                nablaNW = imfilter(diff_im,hNW,'conv','symmetric');

                if option == 1
                    cN = exp(-(nablaN/kappa).^2);
                    cS = exp(-(nablaS/kappa).^2);
                    cW = exp(-(nablaW/kappa).^2);
                    cE = exp(-(nablaE/kappa).^2);
                    cNE = exp(-(nablaNE/kappa).^2);
                    cSE = exp(-(nablaSE/kappa).^2);
                    cSW = exp(-(nablaSW/kappa).^2);
                    cNW = exp(-(nablaNW/kappa).^2);
                elseif option == 2
                    cN = 1./(1 + (nablaN/kappa).^2);
                    cS = 1./(1 + (nablaS/kappa).^2);
                    cW = 1./(1 + (nablaW/kappa).^2);
                    cE = 1./(1 + (nablaE/kappa).^2);
                    cNE = 1./(1 + (nablaNE/kappa).^2);
                    cSE = 1./(1 + (nablaSE/kappa).^2);
                    cSW = 1./(1 + (nablaSW/kappa).^2);
                    cNW = 1./(1 + (nablaNW/kappa).^2);
                end
                diff_im = diff_im + ...
                    delta_t*(...
                    (1/(dy^2))*cN.*nablaN + (1/(dy^2))*cS.*nablaS + ...
                    (1/(dx^2))*cW.*nablaW + (1/(dx^2))*cE.*nablaE + ...
                    (1/(dd^2))*cNE.*nablaNE + (1/(dd^2))*cSE.*nablaSE + ...
                    (1/(dd^2))*cSW.*nablaSW + (1/(dd^2))*cNW.*nablaNW );
            end
        end
    end
end