%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   DSeg software is currently under active development.We welcome comments
%   and contributions to the documentation and code of DSeg to help  us 
%   improve the implementation.
%
%	Related information can be found in the paper :
%
%   DSeg: A dynamic segmentation program to extract backbone patterns for 
%   filamentous bacteria and hyphae structures, March 2018   
%
%   Hanqing Zhang,Niklas Soderholm, Linda Sandblad, Krister Wiklund,
%   Magnus Andersson*
%   * Corresponding author
%
%  Contact information:  Hanqing Zhang :  hanqing.zhang@umu.se
%                        Magnus Andersson : magnus.andersson@umu.se
% 
%  GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
% 
%  Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>
%  Everyone is permitted to copy and distribute verbatim copies
%  of this license document, but changing it is not allowed.
function varargout = DriftCorrection(varargin)
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @DriftCorrection_OpeningFcn, ...
    'gui_OutputFcn',  @DriftCorrection_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% Init
function DriftCorrection_OpeningFcn(hObject, eventdata, handles, varargin)
handles.output = hObject;
handles.InputOperator=loadmedia;
handles.DIHMOperator=dhm_reconstruction;
% Update handles structure
handles=updateCreateHolographyVideoGUI(handles);
handles=disableButtoms(handles);
handles=DisableALLResults(handles);
guidata(hObject, handles);
function varargout = DriftCorrection_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;
function handles=updateCreateHolographyVideoGUI(handles)
set(handles.Edit_ConversionFactor,'String',num2str(handles.DIHMOperator.ReconstructConfig.PixelSpacing*1e9,'%10.2f'));
set(handles.Edit_TimeInterval,'String',num2str(handles.DIHMOperator.SystemConfig.TimeInterval,'%10.2f'));
set(handles.Edit_NumofMeasurement,'String',num2str(handles.DIHMOperator.SystemConfig.GausLayers,'%10.0f'));
if(handles.DIHMOperator.SystemConfig.DriftCorrectionActive==1)
    set(handles.Checkbox_StageDrift,'Value',1);
else
    set(handles.Checkbox_StageDrift,'Value',0);
end
handles=UpateExtensionValue(handles);

% - Release/Disable parameters
function handles=disableButtoms(handles)
set(handles.Buttom_SelectReferenceTemplate,'Enable','off');
set(handles.Buttom_DriftCorrection,'Enable','off');
set(handles.Buttom_plotdrift,'Enable','off');
set(handles.Edit_SelectedFrame,'Enable','off');
set(handles.Slider_frameNum,'Enable','off');
function handles=DisableALLResults(handles)
handles.DIHMOperator.SystemConfig.TemplateROI=[];
handles.DIHMOperator.SystemConfig.Results=[];
% Clear axes
cla(handles.axes_createvideos);
cla(handles.axes_Template);

% Load all videos in the root path of videos with target format
function Buttom_LoadPath_Holography_Callback(hObject, eventdata, handles)
pathname = uigetdir('','Select a path');
if  isempty(pathname) || length(pathname)<2
    disp('Cancled.');
    set(handles.TEXT_loadpath,'String',[]);
    return;
else
    disp(['User selected ', pathname]);
end
handles.InputOperator.LoadMultipleFiles(pathname,handles.DIHMOperator.SystemConfig.Extern);
handles=updateSelectPath(handles,pathname);
handles=UpdateLoadVideo(handles);
guidata(hObject, handles);
function handles=updateSelectPath(handles,pathname)
handles=updateFilelist(handles);
set(handles.TEXT_loadpath,'String',pathname);
function handles=updateFilelist(handles)
file_names={};
for i=1:length(handles.InputOperator.file_list)
    [~,file_names{i},~]=fileparts(handles.InputOperator.file_list{i});
end
set(handles.Listbox_inputvideos,'String',file_names);
set(handles.Listbox_inputvideos, 'Value', 1);

function PopMenu_extension_Callback(hObject, eventdata, handles)
handles=UpateExtensionValue(handles);
guidata(hObject, handles);
function PopMenu_extension_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
function handles=UpateExtensionValue(handles)
extern_value=get(handles.PopMenu_extension,'Value');
switch extern_value
    case 1
        handles.DIHMOperator.SystemConfig.Extern='.avi';
    case 2
        handles.DIHMOperator.SystemConfig.Extern='.mp4';
    case 3
        handles.DIHMOperator.SystemConfig.Extern='.mov';
    case 4
        handles.DIHMOperator.SystemConfig.Extern='.mpg';
    case 5
        handles.DIHMOperator.SystemConfig.Extern='.mpeg';
    case 6
        handles.DIHMOperator.SystemConfig.Extern='.wmv';
    case 7
        handles.DIHMOperator.SystemConfig.Extern='.mov';
    case 8
        handles.DIHMOperator.SystemConfig.Extern='.mj2';
    otherwise
        handles.DIHMOperator.SystemConfig.Extern='.avi';
end

function Listbox_inputvideos_Callback(hObject, eventdata, handles)
handles=UpdateLoadVideo(handles);
guidata(hObject, handles);
function Listbox_inputvideos_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Load a single video
function handles=UpdateLoadVideo(handles)
if(~isempty(handles.InputOperator.file_list))
    file_list_id=get(handles.Listbox_inputvideos,'Value');
    addr=handles.InputOperator.file_list{file_list_id};
    handles.InputOperator.Load(addr);
    handles=EnableLoadVideo(handles);
else
    handles=DisableLoadVideo(handles);
end
function handles=EnableLoadVideo(handles)
set(handles.Buttom_SelectReferenceTemplate,'Enable','on');
set(handles.Edit_SelectedFrame,'Enable','on');
set(handles.Slider_frameNum,'Enable','on');
handles=updateSlideFrameNum(handles);
handles=updateFrameSelected(handles);
PresentFrame(handles);
function handles=updateSlideFrameNum(handles)
set(handles.Slider_frameNum,'Max',handles.InputOperator.TotalFrames);
set(handles.Slider_frameNum,'Min',1);
set(handles.Slider_frameNum,'Value',handles.InputOperator.FrameNumber);
function handles=updateFrameSelected(handles)
set(handles.Edit_SelectedFrame,'String',num2str(handles.InputOperator.FrameNumber,'%10.0f'));

function PresentFrame(handles)
axh=handles.axes_createvideos;
imshow(handles.InputOperator.CurrentFrame,'Parent',axh);
if(~isempty(handles.DIHMOperator.SystemConfig.Template))
    axh=handles.axes_Template;
    imshow(handles.DIHMOperator.SystemConfig.Template,'Parent',axh);
end
function handles=DisableLoadVideo(handles)
set(handles.Buttom_SelectReferenceTemplate,'Enable','off');
set(handles.Edit_SelectedFrame,'Enable','off');
set(handles.Slider_frameNum,'Enable','off');

% Set frame number
function Slider_frameNum_Callback(hObject, eventdata, handles)
CheckValue=round(get(hObject,'Value'));
if(CheckValue>=1 && CheckValue<=handles.InputOperator.TotalFrames)
    handles.InputOperator.SetFrameNumber(CheckValue);
end
handles=updateSlideFrameNum(handles);
handles=updateFrameSelected(handles);
PresentFrame(handles);
function Slider_frameNum_CreateFcn(hObject, eventdata, handles)
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end
function Edit_SelectedFrame_Callback(hObject, ~, handles)
Numstring=get(hObject,'String');
if(isempty(Numstring))
    set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.SelectedFrame));
    guidata(hObject, handles);
    return
end
[FrameNumber, status] = str2num(Numstring);
if(status==0)
    set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.SelectedFrame));
    guidata(hObject, handles);
    return
end
if(FrameNumber>0 && FrameNumber<=handles.InputOperator.TotalFrames)
    handles.DIHMOperator.SystemConfig.SelectedFrame=round(FrameNumber);
    handles.InputOperator.SetFrameNumber(FrameNumber);
    handles=updateSlideFrameNum(handles);
    PresentFrame(handles);
else
    set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.SelectedFrame,'%10.0f'));
    return
end
set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.SelectedFrame,'%10.0f'));
guidata(hObject, handles);
function Edit_SelectedFrame_CreateFcn(hObject, eventdata, handles)
% hObject    handle to Edit_SelectedFrame (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% Set a template
function Buttom_SelectReferenceTemplate_Callback(hObject, eventdata, handles)
handles.DIHMOperator.SystemConfig.TemplateROI=selectROI(handles.InputOperator.CurrentFrame,handles.axes_createvideos);
if(~isempty(handles.DIHMOperator.SystemConfig.TemplateROI))
    handles.DIHMOperator.SystemConfig.SelectedFrame=handles.InputOperator.FrameNumber;
    handles=updateTemplateImage(handles);
    handles=UpdateEnableTemplate(handles);
end
guidata(hObject, handles);
function handles=updateTemplateImage(handles)
handles.InputOperator.SetFrameNumber(handles.InputOperator.FrameNumber);
DiffractionPatternAtFocus=handles.DIHMOperator.Preprocess(handles.InputOperator.CurrentFrame);
axh=handles.axes_Template;
imshow(DiffractionPatternAtFocus,'Parent',axh);
function handles=UpdateEnableTemplate(handles)
set(handles.Buttom_DriftCorrection,'Enable','on');

% system parameters
function Edit_ConversionFactor_Callback(hObject, eventdata, handles)
Numstring=get(hObject,'String');
if(isempty(Numstring))
    set(hObject,'String',num2str(handles.DIHMOperator.ReconstructConfig.PixelSpacing*1e9));
    guidata(hObject, handles);
    return
end
[PixelSpacing, status] = str2num(Numstring);
if(status==0)
    set(hObject,'String',num2str(handles.DIHMOperator.ReconstructConfig.PixelSpacing*1e9));
    guidata(hObject, handles);
    return
end
% Update Parameters (Configuration & instance)
PixelSpacing=PixelSpacing*1e-9;
handles.DIHMOperator.SetConfig('PixelSpacing',PixelSpacing);
set(hObject,'String',num2str(obj.SystemConfig.StepTimeInterval,'%10.2f'));
guidata(hObject, handles);
function Edit_ConversionFactor_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
function Edit_TimeInterval_Callback(hObject, eventdata, handles)
Numstring=get(hObject,'String');
if(isempty(Numstring))
    set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.TimeInterval));
    guidata(hObject, handles);
    return
end
[TimeInterval, status] = str2num(Numstring);
if(status==0)
    set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.TimeInterval));
    guidata(hObject, handles);
    return
end
% Update Parameters (Configuration & instance)
if(TimeInterval>0)
    handles.DIHMOperator.SetConfig('TimeInterval',TimeInterval);
end
set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.TimeInterval,'%10.2f'));
guidata(hObject, handles);
function Edit_TimeInterval_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
function Edit_NumofMeasurement_Callback(hObject, eventdata, handles)
Numstring=get(hObject,'String');
if(isempty(Numstring))
    set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.GausLayers,'%10.0f'));
    guidata(hObject, handles);
    return
end
[Measurements, status] = str2num(Numstring);
if(status==0)
    set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.GausLayers,'%10.0f'));
    guidata(hObject, handles);
    return
end
% Update Parameters (Configuration & instance)
if(round(Measurements)>0)
    handles.DIHMOperator.SetConfig('GausLayers',round(Measurements));
end
set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.GausLayers,'%10.0f'));
guidata(hObject, handles);
function Edit_NumofMeasurement_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% Drift
function Buttom_DriftCorrection_Callback(hObject, ~, handles)
initialFile=get(handles.Listbox_inputvideos, 'Value');
handles=UpdateDrift(handles);
set(handles.Listbox_inputvideos, 'Value',initialFile);
handles.InputOperator.SetFrameNumber(handles.InputOperator.FrameNumber);
PresentFrame(handles);
guidata(hObject, handles);
function handles=UpdateDrift(handles)
% *(use image pattern at focus to find drift)
handles.InputOperator.SetFrameNumber(handles.DIHMOperator.SystemConfig.SelectedFrame);
ComparingImage=handles.DIHMOperator.Preprocess(handles.InputOperator.CurrentFrame);
layers=handles.DIHMOperator.SystemConfig.GausLayers;
ComparingImageLayers=zeros([size(ComparingImage) layers]);
for i=1:layers
    ComparingImageLayers(:,:,i) = double(imgaussfilt(uint8(255*ComparingImage),0.8+0.8*(i-1)))/255; 
end
%%%
frame_number_list=handles.DIHMOperator.SystemConfig.SelectedFrame:1:handles.InputOperator.TotalFrames;
driftValues=zeros(length(frame_number_list),3,layers);
for i=1:length(frame_number_list)
    handles.InputOperator.SetFrameNumber(frame_number_list(i));
    DiffractionPatternAtTest=handles.DIHMOperator.Preprocess(handles.InputOperator.CurrentFrame);
    % Present image
    handles=updateSlideFrameNum(handles);
    handles=updateFrameSelected(handles);
    PresentFrame(handles);
    for j=1:layers
        %
        IntensityResult=handles.DIHMOperator.ImageTranslation_Intensity(DiffractionPatternAtTest,ComparingImageLayers(:,:,j),0);
        if(frame_number_list(i)==handles.DIHMOperator.SystemConfig.SelectedFrame)
            IntensityResultX_offset=IntensityResult.locx*handles.DIHMOperator.ReconstructConfig.PixelSpacing*1e6;
            IntensityResultY_offset=IntensityResult.locy*handles.DIHMOperator.ReconstructConfig.PixelSpacing*1e6;
        end
        driftValues(i,:,j)=[IntensityResult.locx*handles.DIHMOperator.ReconstructConfig.PixelSpacing*1e6-IntensityResultX_offset ...
                          IntensityResult.locy*handles.DIHMOperator.ReconstructConfig.PixelSpacing*1e6-IntensityResultY_offset ...
                          0];
    end
end
handles.DIHMOperator.SystemConfig.Results.DriftValues=driftValues;
handles.DIHMOperator.SystemConfig.Results.MeanDrift=mean(driftValues,3);
handles.DIHMOperator.SystemConfig.Results.StdevDrift=std(driftValues,1,3);
handles.DIHMOperator.SystemConfig.DriftCorrectionActive=1;
handles=updateDriftButtom(handles);
function handles=updateDriftButtom(handles)
set(handles.Buttom_plotdrift,'Enable','on');
% Show drift and remove outliers
function Buttom_plotdrift_Callback(hObject, eventdata, handles)
if(~isempty(handles.DIHMOperator.SystemConfig.Results))
    Xdata=handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,1);
    Ydata=handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,2);
    Zdata=handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,3);
    Xstderr=handles.DIHMOperator.SystemConfig.Results.StdevDrift(:,1);
    Ystderr=handles.DIHMOperator.SystemConfig.Results.StdevDrift(:,2);
    Zstderr=handles.DIHMOperator.SystemConfig.Results.StdevDrift(:,3);
    %
    Tdata=1:handles.DIHMOperator.SystemConfig.TimeInterval:(length(Xdata))*handles.DIHMOperator.SystemConfig.TimeInterval;
    FigHandle=figure;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    F_Width = 26.5; % cm -- IEEE single column width (210-14.32*2-4.22)/3 = 59.04mm (3 columns in a row)
    F_Height = 9; % cm
    p_Width = 8; % cm -- IEEE single column width (210-14.32*2-4.22)/3 = 88.57mm
    p_Height = 8; % cm
    OuterMargin=0.3;
    TextMargin=1;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    shift_horizontal=8.5;
    %
    p_label_Letter_HeightShift=p_Height+0.5;
    p_label_Letter_WidthShift=0.5;  
    p_label_Letter_Width=0.2;
    p_label_Letter_Height=0.2; 
    
    TextWidthShift=0.5;
    TextHightShift=1;
    %%%%%%%%%%%%%%%%%%%%%%%
    shift_x=0;
    shift_y=0;
    %%%%%%%%%%%%%%%%%
    %
    set(FigHandle,'Color','white')
    set(FigHandle,'Unit','centimeters')
    set(FigHandle,'Position',[10 10 F_Width+OuterMargin F_Height+OuterMargin])
    set(FigHandle,'Papertype','a4')
    set(FigHandle,'PaperPosition',[0 0 F_Width+OuterMargin F_Height+OuterMargin]);
    %
    axes_handle=axes('Parent',FigHandle,...
        'Units','centimeters',...
        'OuterPosition',[OuterMargin+shift_x OuterMargin+shift_y p_Width-OuterMargin p_Height-OuterMargin],...
        'Position',[TextMargin+OuterMargin+shift_x TextMargin+OuterMargin+shift_y p_Width-TextMargin-OuterMargin p_Height-TextMargin-OuterMargin],...
        'FontUnits','points',...
        'FontSize',10);
    hold on;
    hplot(1)=plot(Tdata,Xdata,'-sg','Parent',axes_handle);
    hplot(2)=plot(Tdata,Ydata,'-+r','Parent',axes_handle);
    xlabel(axes_handle,'Time (min)')
    ylabel(axes_handle,'Displacement (\mum)');
    
    ErrorRegionsX1=Xdata+Xstderr;
    ErrorRegionsX2=Xdata-Xstderr;
    fill_plot(Tdata',flipud(ErrorRegionsX1),ErrorRegionsX2,axes_handle,[0.3 0.8 0.3]);
    ErrorRegionsY1=Ydata+Ystderr;
    ErrorRegionsY2=Ydata-Ystderr;
    fill_plot(Tdata',flipud(ErrorRegionsY1),ErrorRegionsY2,axes_handle,[0.8 0.3 0.3]);
    
    axes_handle_label1=axes('Parent',FigHandle,...
        'Units','centimeters',...
        'OuterPosition',[p_label_Letter_WidthShift+shift_x p_label_Letter_HeightShift+shift_y p_label_Letter_Width p_label_Letter_Height],...
        'Position',[p_label_Letter_WidthShift+shift_x p_label_Letter_HeightShift+shift_y p_label_Letter_Width p_label_Letter_Height],...
        'FontUnits','points',...
        'FontSize',10);
    set(axes_handle_label1, 'box','off','XTickLabel',[],'XTick',[],'YTickLabel',[],'YTick',[])
    color = get(FigHandle,'Color');
    set(axes_handle_label1,'XColor',color,'YColor',color,'TickDir','out')
    text(TextWidthShift,TextHightShift,'A)','FontSize',10);
    legend(axes_handle,hplot(1:end),'X','Y ','Location','NorthOutside','Box','off','Orientation','horizontal')
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Mark outliers and replace with interplated value
    Xdata=findOutlier(Xdata,handles.DIHMOperator.SystemConfig.OutlierValue);
    Ydata=findOutlier(Ydata,handles.DIHMOperator.SystemConfig.OutlierValue);
    %
    shift_x=1*shift_horizontal;
    shift_y=0;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    set(FigHandle,'Color','white')
    set(FigHandle,'Unit','centimeters')
    set(FigHandle,'Position',[10 10 F_Width+OuterMargin F_Height+OuterMargin])
    set(FigHandle,'Papertype','a4')
    set(FigHandle,'PaperPosition',[0 0 F_Width+OuterMargin F_Height+OuterMargin]);
    %
    axes_handle2=axes('Parent',FigHandle,...
        'Units','centimeters',...
        'OuterPosition',[OuterMargin+shift_x OuterMargin+shift_y p_Width-OuterMargin p_Height-OuterMargin],...
        'Position',[TextMargin+OuterMargin+shift_x TextMargin+OuterMargin+shift_y p_Width-TextMargin-OuterMargin p_Height-TextMargin-OuterMargin],...
        'FontUnits','points',...
        'FontSize',10,...
        'Tag','PlotNoOutliers');
    hold on;
    hplot(1)=plot(Tdata,Xdata,'-sg','Parent',axes_handle2);
    hplot(2)=plot(Tdata,Ydata,'-+r','Parent',axes_handle2);
    xlabel(axes_handle2,'Time (min)')
    ylabel(axes_handle2,{'Cubic spline interpolated data','with outliers removed (\mum)'});
    legend(axes_handle2,hplot(1:end),'X ','Y ','Location','NorthOutside','Box','off','Orientation','horizontal');
    
    axes_handle_label2=axes('Parent',FigHandle,...
        'Units','centimeters',...
        'OuterPosition',[p_label_Letter_WidthShift+shift_x p_label_Letter_HeightShift+shift_y p_label_Letter_Width p_label_Letter_Height],...
        'Position',[p_label_Letter_WidthShift+shift_x p_label_Letter_HeightShift+shift_y p_label_Letter_Width p_label_Letter_Height],...
        'FontUnits','points',...
        'FontSize',10);
    set(axes_handle_label2, 'box','off','XTickLabel',[],'XTick',[],'YTickLabel',[],'YTick',[])
    color = get(FigHandle,'Color');
    set(axes_handle_label2,'XColor',color,'YColor',color,'TickDir','out')
    text(TextWidthShift,TextHightShift,'B)','FontSize',10);

    handles.Button_Update_outlierlevel=uicontrol('Parent',FigHandle,...
        'Style','pushbutton',...
        'String','Apply',...
        'Units','centimeters',...
        'Position',[p_label_Letter_WidthShift+shift_x+7 p_label_Letter_HeightShift+shift_y p_label_Letter_Width+1 p_label_Letter_Height+0.5],...
        'Callback', {@Buttom_Apply_outlierlevel_callback,handles});
    
    handles.Button_Update_outlierlevel=uicontrol('Parent',FigHandle,...
        'Style','pushbutton',...
        'String','Update threshold value for outliers',...
        'Units','centimeters',...
        'Position',[p_label_Letter_WidthShift+shift_x+2.2 p_label_Letter_HeightShift+shift_y p_label_Letter_Width+4.5 p_label_Letter_Height+0.5],...
        'Callback', {@Buttom_Update_outlierlevel_callback,handles});
    
    handles.Edit_outlierlevel=uicontrol('Parent',FigHandle,...
        'Style','edit',...
        'String',num2str(handles.DIHMOperator.SystemConfig.OutlierValue),...
        'Units','centimeters',...
        'Position',[p_label_Letter_WidthShift+shift_x+1 p_label_Letter_HeightShift+shift_y p_label_Letter_Width+1 p_label_Letter_Height+0.5],...
        'Callback', {@Edit_outlierlevel_callback,handles});

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    shift_x=2*shift_horizontal;
    shift_y=0;
    %
    axes_handle3=axes('Parent',FigHandle,...
        'Units','centimeters',...
        'OuterPosition',[OuterMargin+shift_x OuterMargin+shift_y p_Width-OuterMargin p_Height-OuterMargin],...
        'Position',[TextMargin+OuterMargin+shift_x TextMargin+OuterMargin+shift_y p_Width-TextMargin-OuterMargin p_Height-TextMargin-OuterMargin],...
        'FontUnits','points',...
        'FontSize',10,...
        'Tag','Plot3D');
    hold on;
    scatter3(Ydata,Xdata,Tdata,20*ones(1,length(Xdata)), Tdata,'filled','Parent',axes_handle3);
    line(Ydata,Xdata,Tdata,'Parent',axes_handle3);
    view([0 90]);
    set(axes_handle3,'YDir','normal');
    set(axes_handle3, 'Tag','Plot3D');
    colormap(winter);
    hcolor =colorbar;
    hcolor.Label.String = 'Time (min)';
    xlabel(axes_handle3,'Horizontal direction (\mum)');
    ylabel(axes_handle3,'Vertical direction (\mum)');
    
    axes_handle_label3=axes('Parent',FigHandle,...
        'Units','centimeters',...
        'OuterPosition',[p_label_Letter_WidthShift+shift_x p_label_Letter_HeightShift+shift_y p_label_Letter_Width p_label_Letter_Height],...
        'Position',[p_label_Letter_WidthShift+shift_x p_label_Letter_HeightShift+shift_y p_label_Letter_Width p_label_Letter_Height],...
        'FontUnits','points',...
        'FontSize',10);
    set(axes_handle_label3, 'box','off','XTickLabel',[],'XTick',[],'YTickLabel',[],'YTick',[])
    color = get(FigHandle,'Color');
    set(axes_handle_label3,'XColor',color,'YColor',color,'TickDir','out')
    text(TextWidthShift,TextHightShift,'C)','FontSize',10);
end
function Buttom_Apply_outlierlevel_callback(hObject, eventdata, handles)
Xdata=handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,1);
Ydata=handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,2);
Zdata=handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,3);
Xdata=findOutlier(Xdata,handles.DIHMOperator.SystemConfig.OutlierValue);
Ydata=findOutlier(Ydata,handles.DIHMOperator.SystemConfig.OutlierValue);
Zdata=findOutlier(Zdata,handles.DIHMOperator.SystemConfig.OutlierValue);
handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,1)=Xdata;
handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,2)=Ydata;
handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,3)=Zdata;
set(hObject,'Enable','off');
guidata(hObject, handles);
function Buttom_Update_outlierlevel_callback(hObject, eventdata,handles)
% Update Parameters (Configuration & instance)
FigHandle=get(hObject,'Parent');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
F_Width = 26.5; % cm -- IEEE single column width (210-14.32*2-4.22)/3 = 59.04mm (3 columns in a row)
F_Height = 9; % cm
p_Width = 8; % cm -- IEEE single column width (210-14.32*2-4.22)/3 = 88.57mm
p_Height = 8; % cm
OuterMargin=0.3;
TextMargin=1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
shift_horizontal=8.5;
%
p_label_Letter_HeightShift=p_Height+0.5;
p_label_Letter_WidthShift=0.5;

p_label_Letter_Width=0.2;
p_label_Letter_Height=0.2;

TextWidthShift=0.5;
TextHightShift=1;

p_legendWidthshift=-0.5;
p_legendHeightshift=6.5;
p_legendFontSize=10;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Xdata=handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,1);
Ydata=handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,2);
Zdata=handles.DIHMOperator.SystemConfig.Results.MeanDrift(:,3);
Tdata=1:handles.DIHMOperator.SystemConfig.TimeInterval:(length(Xdata))*handles.DIHMOperator.SystemConfig.TimeInterval;
Xdata=findOutlier(Xdata,handles.DIHMOperator.SystemConfig.OutlierValue);
Ydata=findOutlier(Ydata,handles.DIHMOperator.SystemConfig.OutlierValue);
Zdata=findOutlier(Zdata,handles.DIHMOperator.SystemConfig.OutlierValue); 
shift_x=1*shift_horizontal;
    shift_y=0;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    set(FigHandle,'Color','white')
    set(FigHandle,'Unit','centimeters')
    set(FigHandle,'Position',[10 10 F_Width+OuterMargin F_Height+OuterMargin])
    set(FigHandle,'Papertype','a4')
    set(FigHandle,'PaperPosition',[0 0 F_Width+OuterMargin F_Height+OuterMargin]);
    %
    i_delete=0;
    for i=1:length(FigHandle.Children)
         if(strcmp(FigHandle.Children(i).Tag,'PlotNoOutliers'))
                i_delete=i;
         end
    end
    if(i_delete~=0)
        delete(FigHandle.Children(i_delete));
    end
    axes_handle2=axes('Parent',FigHandle,...
        'Units','centimeters',...
        'OuterPosition',[OuterMargin+shift_x OuterMargin+shift_y p_Width-OuterMargin p_Height-OuterMargin],...
        'Position',[TextMargin+OuterMargin+shift_x TextMargin+OuterMargin+shift_y p_Width-TextMargin-OuterMargin p_Height-TextMargin-OuterMargin],...
        'FontUnits','points',...
        'FontSize',10,...
        'Tag','PlotNoOutliers');
    hold on;
    hplot(1)=plot(Tdata,Xdata,'-sg','Parent',axes_handle2);
    hplot(2)=plot(Tdata,Ydata,'-+r','Parent',axes_handle2);
    xlabel(axes_handle2,'Time (min)')
    ylabel(axes_handle2,{'Cubic spline interpolated data','with outliers removed (\mum)'});
    legend(axes_handle2,hplot(1:end),'X','Y','Location','NorthOutside','Box','off','Orientation','horizontal');
    
    % Update 3d plot
    for  i_find=1:length(FigHandle.Children)
         if(strcmp(FigHandle.Children(i_find).Tag,'Plot3D'))
                axes_handle3=FigHandle.Children(i_find);
                hold off;
                scatter3(Ydata,Xdata,Tdata,20*ones(1,length(Xdata)), Tdata,'filled','Parent',axes_handle3);hold on;
                line(Ydata,Xdata,Tdata,'Parent',axes_handle3);hold on;
                view([0 90]);
                set(axes_handle3,'YDir','normal');
         end
    end

guidata(hObject, handles);
function Edit_outlierlevel_callback(hObject, eventdata, handles)
Numstring=get(hObject,'String');
if(isempty(Numstring))
    set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.OutlierValue));
    guidata(hObject, handles);
    return
end
[Num, status] = str2num(Numstring);
if(status==0)
    set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.OutlierValue));
    guidata(hObject, handles);
    return
end
handles.DIHMOperator.SystemConfig.OutlierValue=Num;
guidata(hObject, handles);

function Checkbox_StageDrift_Callback(hObject, eventdata, handles)
CheckValue=get(hObject,'Value');
if(CheckValue==1)
    handles.DIHMOperator.SystemConfig.DriftCorrectionActive=1;
else
    handles.DIHMOperator.SystemConfig.DriftCorrectionActive=0;
end
guidata(hObject, handles);

% Create & Enhancement
function Buttom_CreateVideo_Callback(hObject, eventdata, handles)
frame_number_list=handles.DIHMOperator.SystemConfig.SelectedFrame:1:handles.InputOperator.TotalFrames;

if(handles.DIHMOperator.SystemConfig.DriftCorrectionActive==1)
    MeanDrift=handles.DIHMOperator.SystemConfig.Results.MeanDrift;
else
    MeanDrift=zeros(length(frame_number_list),3);
end

VideoSavePrefix='DC_';
handles.InputOperator.name;
FileName=strcat(VideoSavePrefix,handles.InputOperator.name,handles.InputOperator.extension);
FolderSave='Created Videos';
FolderPath=strcat('./',FolderSave,'/');
if(exist(FolderSave) ~= 7)
    mkdir(FolderPath);
end

writerObj = VideoWriter(strcat(FolderPath,FileName));
writerObj.FrameRate =30;
open(writerObj);
for i=1:length(frame_number_list)
    % Load video and find focus frame
    re_sample_x=repmat((1:handles.InputOperator.Height)',1,handles.InputOperator.Width)+MeanDrift(i,1)/handles.DIHMOperator.ReconstructConfig.PixelSpacing*1e-6;
    re_sample_y=repmat(1:handles.InputOperator.Width,handles.InputOperator.Height,1)+MeanDrift(i,2)/handles.DIHMOperator.ReconstructConfig.PixelSpacing*1e-6;

    handles.InputOperator.SetFrameNumber(frame_number_list(i));
    ImageInput=handles.DIHMOperator.Preprocess(handles.InputOperator.CurrentFrame,2);

    if(frame_number_list(i)~=handles.DIHMOperator.SystemConfig.SelectedFrame)
            IntensityResult = interp2(ImageInput,re_sample_y,re_sample_x,'cubic',0);
    else
            IntensityResult = ImageInput;
    end
    PresentDriftFreeFrame(handles,IntensityResult);
    handles=updateSlideFrameNum(handles);
    handles=updateFrameSelected(handles);
    writeVideo(writerObj,uint8(255*IntensityResult));
end
disp('done.')
close(writerObj);

function SaveEnhancedImages(handles,image,id,ImageSavePrefix)
ImageSaveName=handles.InputOperator.name;
ImageSavePostfix=strcat('_',num2str(id));
currentFolder =strcat(pwd,'\Saved Images\');
mkdir(currentFolder);
FileName=strcat(currentFolder,ImageSavePrefix,ImageSaveName,ImageSavePostfix,'.png');
imwrite(image,FileName);
function PresentDriftFreeFrame(handles,image_in)
axh=handles.axes_createvideos;
imshow(image_in,'Parent',axh);
function Edit_FocusOffset_Callback(hObject, eventdata, handles)
Numstring=get(hObject,'String');
if(isempty(Numstring))
    set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.FocusOffset*1e6,'%10.2f'));
    guidata(hObject, handles);
    return
end
[FocusOffset, status] = str2num(Numstring);
if(status==0)
    set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.FocusOffset*1e6,'%10.2f'));
    guidata(hObject, handles);
    return
end
% Update Parameters (Configuration & instance)
handles.DIHMOperator.SystemConfig.FocusOffset=FocusOffset*1e-6;
set(hObject,'String',num2str(handles.DIHMOperator.SystemConfig.FocusOffset*1e6,'%10.2f'));
guidata(hObject, handles);
function Edit_FocusOffset_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function Buttom_VideoFileDelete_Callback(hObject, eventdata, handles)
id=get(handles.Listbox_inputvideos, 'Value');
if(~isempty(handles.InputOperator.file_list) && id~=0)
    list_length=length(handles.InputOperator.file_list);
    if(list_length==1 && id==1)
    else
        handles.InputOperator.file_list{id}={};
        if(id-1>1)
            set(handles.Listbox_inputvideos, 'Value',id-1);
        end
    end
end
handles.InputOperator.file_list=handles.InputOperator.file_list(~cellfun('isempty',handles.InputOperator.file_list));
handles=updateFilelist(handles);
handles=UpdateLoadVideo(handles);
guidata(hObject, handles);


% --------------------------------------------------------------------
function Help_Callback(hObject, eventdata, handles)
set(handles.TEXT_loadpath,'String','Select a folder using [Load path] -> Select an video file -> Select a small template in the first frame -> Click [Estimate drift] -> Check drift result by clicking [Show drift] -> Create video with drift correction');
