function Result=findExtremValues3D(Matrix,Mode,KernelSize,Debug)
% Find the position of extreme value in 3d matrix
% Inputs: Matrix 
%         Mode: 1 = maximum 2 = minimum value
%         KernelSize: parabolic fitting kernel. 0= cancel
% Outputs: Results
% By Hanqing Zhang : hanqing.zhang@umu.se 2017/10/25
sd_list=[1];
if(Mode==1)
    [peak_loc_X,peak_loc_Y,peak_loc_Z]=ind2sub(size(Matrix),find(Matrix==max(Matrix(:)),1));     
elseif(Mode==2)
    MatrixP=permute(Matrix,[1,3,2]);
    [X,~,Z]=size(MatrixP);
    M_kALL=zeros(X,Z);
    for i=1:length(sd_list)
        M_k1=zeros(size(MatrixP));
        MaxV=zeros(X,Z);
        sd=sd_list(i); 
        w=2*round(sd*3)+1;
        x = meshgrid(-floor(w/2):floor(w/2),1);
        G = exp(-(x.^2)/(2*pi*(sd^2)));
        G_norm=G/sum(G(:));
        Gx = x.*G_norm/(sd^2);
        for i_z=1:Z
            for i_x=1:X
                 M_k1(i_x,:,i_z) = filter(G_norm,1,MatrixP(i_x,:,i_z));
                 MaxV(i_x,i_z)=max(M_k1(i_x,:,i_z));
            end
        end
        % Direction
        M_k1(M_k1<=0)=0;
        % Feature values
        MLevel=std(MaxV(:));
        MLevelHigh=max(M_k1(:));
        % Strong signal
        Blob=(M_k1>=(MLevelHigh-MLevel));
        Intensity2d=sum(Blob.*MatrixP,2);
        Intensity2d=permute(Intensity2d,[1,3,2]);
        M_kALL=M_kALL+Intensity2d;
    end
    % Center-of-Mass
    template_corr_Z=1:Z;
    template_corr_X=1:X;
    corr_col = double(repmat(template_corr_Z, [X 1]));
    corr_row = double(repmat(template_corr_X', [1 Z]));
    total_intensity=sum(M_kALL(:));
    peak_loc_Y=round(sum(sum(corr_col.*M_kALL))/total_intensity);
    peak_loc_X=round(sum(sum(corr_row.*M_kALL))/total_intensity);
    
    Z_Region=1;
    Z_IntensityRegion=Matrix(peak_loc_X-Z_Region:peak_loc_X+Z_Region,peak_loc_Y-Z_Region:peak_loc_Y+Z_Region,:);
    SumIntensityZ=-reshape(sum(sum(Z_IntensityRegion,2),1),1,[]);
    peak_loc_Z=find(SumIntensityZ==max(SumIntensityZ(:)),1);
    
    %[peak_loc_X1,peak_loc_Y1,peak_loc_Z1]=ind2sub(size(M_k1),find(M_k1==max(M_k1(:)),1)) 
    %[peak_loc_X,peak_loc_Y,peak_loc_Z]=ind2sub(size(M_kALL),find(M_kALL==max(M_kALL(:)),1))
else
    [peak_loc_X,peak_loc_Y,peak_loc_Z]=ind2sub(size(Matrix),find(Matrix==min(Matrix(:)),1)); 
end
if(Debug>0)
    figure(Debug);plot(reshape(Matrix(peak_loc_X,peak_loc_Y,:),1,[]));hold on;
end
% Interpolation in x, y and z
fit_r=KernelSize;
if(peak_loc_X-fit_r>0) && (peak_loc_X+fit_r<=size(Matrix,1) && fit_r~=0)
    % parabolic fit
    px=polyfit(1:2*fit_r+1,Matrix(peak_loc_X-fit_r:peak_loc_X+fit_r,peak_loc_Y,peak_loc_Z)',2);
    px_peak=-px(2)/(2*px(1));
    Result.locx=peak_loc_X-(fit_r+1-px_peak);
else
    Result.locx=peak_loc_X;
end
if(peak_loc_Y-fit_r>0) && (peak_loc_Y+fit_r<=size(Matrix,2) && fit_r~=0)
    % parabolic fit
    pz=polyfit(1:2*fit_r+1,Matrix(peak_loc_X,peak_loc_Y-fit_r:peak_loc_Y+fit_r,peak_loc_Z),2);
    pz_peak=-pz(2)/(2*pz(1));
    Result.locy=peak_loc_Y-(fit_r+1-pz_peak);
else
    Result.locy=peak_loc_Y;
end
if(peak_loc_Z-fit_r>0) && (peak_loc_Z+fit_r<=size(Matrix,3) && fit_r~=0)
    % parabolic fit
    pz=polyfit(1:2*fit_r+1,reshape(Matrix(peak_loc_X,peak_loc_Y,peak_loc_Z-fit_r:peak_loc_Z+fit_r),1,[]),2);
    pz_peak=-pz(2)/(2*pz(1));
    Result.locz=peak_loc_Z-(fit_r+1-pz_peak);
else
    Result.locz=peak_loc_Z;
end