%   This file is a part of UMUTracker Project.The UMUTracker is currently under active development.
%	Related information can be found in the paper :
%
%	Hanqing Zhang, Tim Stangner, Krister Wiklund, Alvaro Rodriguez, Magnus Andersson
%	UmUTracker: A versatile MATLAB program for automated particle tracking of 2D light microscopy or 3D digital holography data
%
%	We welcome comments and contributions to the documentation and code of UMUTracker to help us improve the implementation.
%
%   Version: 1.Initial version:  Hanqing Zhang,hanqing.zhang@umu.se
%
classdef loadmedia <  INTERFACE_loadmedia
    properties (SetAccess = protected, GetAccess = public)
        % File Properties
        fullpath='';
        name='';
        path='';
        extension='';
        type='None'; % Image , Video or None
        
        % Media Properties
        Method=[];
        CurrentFrame=[];
        Video_Method='Standard'; % Standard or Vision 
        VideoSrc='';
        TotalFrames=0;
        FrameNumber=0;
        FrameRate=0;
        Width=0;
        Height =0;
        ColorFormat='';
    end
    properties (SetAccess = public, GetAccess = public)
       file_list=[]; 
    end
    
    % Main Functions 
    methods
        function obj=Load(obj,Address,Method)
            if nargin==3
                % Get Address
                AddrProp=obj.SetAddressFile(Address);
                if(strcmp(AddrProp.type,'Image'))
                    MediaProp=obj.GetImage(AddrProp.fullpath);
                elseif(strcmp(AddrProp.type,'Video'))
                    MediaProp=obj.GetVideo(AddrProp.fullpath,Method);
                else
                end
            elseif nargin==2
                AddrProp=obj.SetAddressFile(Address);
                if(strcmp(AddrProp.type,'Image'))
                    MediaProp=obj.GetImage(AddrProp.fullpath);
                elseif(strcmp(AddrProp.type,'Video'))
                    MediaProp=obj.GetVideo(AddrProp.fullpath);
                else
                end
            else
                AddrProp=obj.SetAddressFileUI();
                if(strcmp(AddrProp.type,'Image'))
                    MediaProp=obj.GetImage(AddrProp.fullpath);
                elseif(strcmp(AddrProp.type,'Video'))
                    MediaProp=obj.GetVideo(AddrProp.fullpath);
                else
                end
            end
            proplist_addr = fieldnames(AddrProp); %properties(AddrProp);
            for i=1:length(proplist_addr)
                obj.(proplist_addr{i}) =AddrProp.(proplist_addr{i});
            end
            proplist_media = fieldnames(MediaProp);
            for i=1:length(proplist_media)
                obj.(proplist_media{i}) = MediaProp.(proplist_media{i});
            end
        end
        
        function obj=LoadMultipleImages(obj,Address)
            if nargin==2
                MultipleImageProp=obj.SetAddressFolder(Address);
            else
                MultipleImageProp=obj.SetAddressFolder();
            end
            proplist_media = fieldnames(MultipleImageProp);
            for i=1:length(proplist_media)
                obj.(proplist_media{i}) = MultipleImageProp.(proplist_media{i});
            end
        end
        function obj=LoadMultipleFiles(obj,Path,Extern)
            Addresslist={};
            if nargin==3
                obj.file_list=Func_getpath_allfiles(Addresslist,Path,Extern);
                obj.fullpath=Path;
            else
                error('No enough parameters');
            end
        end
        
        function obj=Step(obj)
            if(obj.FrameNumber<obj.TotalFrames)
                if(strcmp(obj.type,'Video'))
                    if(strcmp(obj.Video_Method,'Standard'))
                        obj.CurrentFrame=readFrame(obj.VideoSrc);
                        obj.FrameNumber=obj.FrameNumber+1;
                    else
                        obj.CurrentFrame=step(obj.VideoSrc);
                        obj.FrameNumber=obj.FrameNumber+1;
                    end
                else
                    if(~isempty(obj.file_list))
                        obj.fullpath=obj.file_list{obj.FrameNumber};
                        [~, ~,obj.extension] = fileparts(obj.fullpath);
                        info = imfinfo(obj.fullpath);
                        obj.ColorFormat= info.ColorType;
                        
                        obj.CurrentFrame=imread(obj.fullpath);
                        obj.FrameNumber=obj.FrameNumber+1;
                        obj.Width=size(obj.CurrentFrame,1);
                        obj.Height=size(obj.CurrentFrame,2);
                        
                        obj.FrameNumber=obj.FrameNumber+1;
                    end
                end
            end
        end
        function obj=SetFrameNumber(obj,NewFrameNumber)
            if(strcmp(obj.type,'Video'))
                if(strcmp(obj.Video_Method,'Standard'))
                    if(NewFrameNumber>=1 && NewFrameNumber<=obj.TotalFrames) % 㿪ʼ-1
                         set(obj.VideoSrc,'CurrentTime',(NewFrameNumber-1)/obj.VideoSrc.FrameRate);
                         obj.CurrentFrame=readFrame(obj.VideoSrc);
                         obj.FrameNumber=NewFrameNumber;
                    end
                end
            else
                if(NewFrameNumber>=1 && NewFrameNumber<=obj.TotalFrames) % 1ʼ
                    obj.fullpath=obj.file_list{NewFrameNumber};
                    [~, ~,obj.extension] = fileparts(obj.fullpath);
                    info = imfinfo(obj.fullpath);
                    obj.ColorFormat= info.ColorType;
                
                    obj.CurrentFrame=imread(obj.fullpath);
                    obj.FrameNumber=NewFrameNumber;
                    obj.Width=size(obj.CurrentFrame,1);
                    obj.Height=size(obj.CurrentFrame,2);
                end
            end
        end
       
        function TF_Result=isDone(obj)
                if(strcmp(obj.Video_Method,'Standard'))
                    if(obj.FrameNumber>=1 && obj.FrameNumber>obj.TotalFrames)
                        TF_Result=true;
                    else
                        TF_Result=false;
                    end
                else
                    TF_Result=isDone(obj.VideoSrc);
                end
        end
        function obj=Reset(obj) % صʼͼƬ,ƵļͲ
            if(strcmp(obj.type,'Video'))
                if(strcmp(obj.Video_Method,'Standard'))
                    set(obj.VideoSrc,'CurrentTime',0);
                    obj.CurrentFrame=readFrame(obj.VideoSrc);
                    obj.FrameNumber=1;
                else
                    reset(obj.VideoSrc);
                    obj.FrameNumber=1;
                end
            else
                obj.fullpath=obj.file_list{1};
                [~, ~,obj.extension] = fileparts(obj.fullpath);
                info = imfinfo(obj.fullpath);
                obj.ColorFormat= info.ColorType;
                
                obj.FrameNumber=1;
                obj.CurrentFrame=imread(obj.fullpath);
                
                obj.Width=size(obj.CurrentFrame,1);
                obj.Height=size(obj.CurrentFrame,2);
                
            end
        end
    end

    % Local functions
    methods (Static)
        function Outputs=SetAddressFile(Address)
            if(ischar(Address))
                Outputs.fullpath=Address;
                [Outputs.path, Outputs.name,Outputs.extension] = fileparts(Outputs.fullpath);
            else
                disp('Wrong inputs for data path');
            end
            if sum(strcmp(Outputs.extension,{'.avi', '.mpg', '.mpeg', '.wmv', '.mp4','.mov','.mj2','.czi'}))>=1
                Outputs.type='Video';
            elseif sum(strcmp(Outputs.extension,{'.gif', '.jpeg','.jpg', '.png', '.tiff', '.bmp','.ras'}))>=1
                Outputs.type='Image';
            else
                Outputs.type='None';
            end
        end
        function Outputs=SetAddressFolder(Address)
            if(nargin>0)
                if(ischar(Address))
                    Outputs.path=Address;
                else
                    Outputs.path=[];
                    disp('Wrong input path...');
                end
            else
                Outputs.path = uigetdir('','Select a path');
            end
            if(sum(Outputs.path==0))
                disp('Cancelled...');
                Outputs.type='None';
                return
            end
            all_files=dir(Outputs.path);
            Outputs.file_list={};
            inc=0;
            for id = 1:length(all_files)
                [~, ~, format_ext] = fileparts(all_files(id).name);
                if sum(strcmp(format_ext,{'.gif', '.jpeg','.jpg', '.png', '.tiff', '.bmp',',ras'}))>=1
                    inc=inc+1;
                    Outputs.file_list{inc}= fullfile(Outputs.path , all_files(id).name);
                    Outputs.extension=format_ext;
                end
            end
            if(inc~=0)
                Outputs.type='Image';
                Outputs.fullpath=Outputs.file_list{1};
                Outputs.name='MultipleImages';
                
                Outputs.VideoSrc=[];
                Outputs.TotalFrames=inc;
                Outputs.CurrentFrame = imread(Outputs.fullpath);
                info = imfinfo(Outputs.fullpath);
                Outputs.FrameNumber=1;
                Outputs.FrameRate=0;
                Outputs.Width=size(Outputs.CurrentFrame,1);
                Outputs.Height=size(Outputs.CurrentFrame,2);
                Outputs.ColorFormat= info.ColorType;
            else
                Outputs.type='None';
            end
        end
        
        function Outputs=SetAddressFileUI()
            [filename, pathname] = uigetfile( ...
                {'*.gif;*.jpeg;*.jpg;*.png;*.tiff;*.bmp;*.ico;*.ras;*.avi;*.mpg;*.mpeg;*.wmv;*.mp4;*.mj2;*.mov',...
                 'Load Media Files (*.gif,*.jpeg,*.jpg,*.png,*.tiff,*.bmp,*.ico,*.ras,*.avi,*.mpg,*.mpeg,*.wmv,*.mp4,*.mj2,*.mov)';
                '*.*',  'All Files (*.*)'}, ...
                'Select a media file');
            if(sum(pathname==0) || sum(filename==0))
                disp('Cancelled...');
                Outputs.type='None';
                return
            else
                Outputs.fullpath=[pathname filename];
                [Outputs.path, Outputs.name,Outputs.extension] = fileparts(Outputs.fullpath);
                if sum(strcmp(Outputs.extension,{'.avi', '.mpg', '.mpeg', '.wmv', '.mp4',',mov','.mj2'}))>=1
                    Outputs.type='Video';
                elseif sum(strcmp(Outputs.extension,{'.gif', '.jpeg','.jpg', '.png', '.tiff', '.bmp',',ras'}))>=1
                    Outputs.type='Image';
                else
                    Outputs.type='None';
                end
            end
        end

        function Outputs=GetImage(Address)
            Outputs.Method='Standard';
            Outputs.VideoSrc=[];
            Outputs.TotalFrames=1;
            Outputs.CurrentFrame = imread(Address);
            info = imfinfo(Address);
            Outputs.FrameNumber=1;
            Outputs.FrameRate=0;
            Outputs.Width=size(Outputs.CurrentFrame,1);
            Outputs.Height=size(Outputs.CurrentFrame,2);
            Outputs.ColorFormat= info.ColorType;
        end
        
        function Outputs=GetVideo(Address,Method)
            if nargin==2 &&(strcmp(Method,'Vision'))
                    Outputs.Method='Vision';
                try
                    Outputs.VideoSrc=vision.VideoFileReader(Address);
                    S = info(Outputs.VideoSrc);
                    Outputs.TotalFrames=0;
                    Outputs.CurrentFrame=step(Outputs.VideoSrc);
                    Outputs.FrameNumber=1;
                    Outputs.FrameRate=S.VideoFrameRate;
                    Outputs.Width=S.VideoSize(1);
                    Outputs.Height=S.VideoSize(2);
                    Outputs.ColorFormat= S.VideoFormat;
                catch
                    error('Cannot load this file using Computer Vision System Toolbox.')
                end
            else
                Outputs.Method='Standard';
                try
                    Outputs.VideoSrc=VideoReader(Address);
                    Outputs.TotalFrames=round(Outputs.VideoSrc.Duration*Outputs.VideoSrc.FrameRate);
                    Outputs.CurrentFrame=readFrame(Outputs.VideoSrc);
                    Outputs.FrameNumber=1;
                    Outputs.FrameRate = Outputs.VideoSrc.FrameRate;
                    Outputs.Width  = Outputs.VideoSrc.Width;
                    Outputs.Height = Outputs.VideoSrc.Height;
                    Outputs.ColorFormat=  Outputs.VideoSrc.VideoFormat;
                catch
                    error('Cannot load this video file.');
                end
            end
        end
        
        
    end
end