%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   DSeg software is currently under active development.We welcome comments
%   and contributions to the documentation and code of DSeg to help  us 
%   improve the implementation.
%
%	Related information can be found in the paper :
%
%   DSeg: A dynamic segmentation program to extract backbone patterns for 
%   filamentous bacteria and hyphae structures, March 2018   
%
%   Hanqing Zhang,Niklas Soderholm, Linda Sandblad, Krister Wiklund,
%   Magnus Andersson*
%   * Corresponding author
%
%  Contact information:  Hanqing Zhang :  hanqing.zhang@umu.se
%                        Magnus Andersson : magnus.andersson@umu.se
% 
%  GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
% 
%  Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>
%  Everyone is permitted to copy and distribute verbatim copies
%  of this license document, but changing it is not allowed.
function varargout = PostAnalysis(varargin)
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @PostAnalysis_OpeningFcn, ...
                   'gui_OutputFcn',  @PostAnalysis_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

function PostAnalysis_OpeningFcn(hObject, eventdata, handles, varargin)
handles.output = hObject;
handles=InitLoad(handles,'Saved Data/Detection');
handles=InitData(handles);
guidata(hObject, handles);
function varargout = PostAnalysis_OutputFcn(hObject, eventdata, handles) 
varargout{1} = handles.output;

function Data=ReadDetection(Path)
files = dir(strcat(Path,'/*.mat'));
if(isempty(files))
    Data=[];
else
    for i=1:length(files)
        Data{i}=load(strcat(files(i).folder,'\',files(i).name));
        Data{i}.Path=strcat(files(i).folder,'\',files(i).name);
    end
end
function handles=InitLoad(handles,Path)
handles.Data=ReadDetection(Path);
set(handles.TEXT_PostAnalysisDataDistribution_LoadSample1_Address,'String',Path);
try
    ImageTemplate=imread(strcat(Path,'/','Template.png'));
    handles.Template=ImageTemplate;
catch
    handles.Template=[];
    disp('No image template available.');
end
function handles=InitData(handles)
%% Update Config
handles.Config.NumOfData=length(handles.Data);
if(handles.Config.NumOfData>0)
    try
        handles.Config.CurrentID=1;
        handles.Config.Resize=handles.Data{1}.Config.Resize;
        handles.Config.PixelSpacing=handles.Data{1}.Config.System.PixelSpacing;
        handles.Config.TimeInterval=handles.Data{1}.Config.System.TimeInterval;
        handles.Config.ROI=handles.Data{1}.Config.ROI;
        handles.Config.Width=handles.Data{1}.Config.Width;
        handles.Config.Height=handles.Data{1}.Config.Height;
        handles.Config.Offset=handles.Data{1}.Config.Offset;
        handles.Config.FilterSize=5;
        handles=ButtonsControl(handles,'On');
    catch
        handles=ButtonsControl(handles,'off');
        error('Data format error.')
    end
else
    handles=ButtonsControl(handles,'off');
    disp('No data files available.');
    return
end
set(handles.List_Objects,'Value',handles.Config.CurrentID);
% Create tempalte image
handles=updateobjectsdata(handles);

if(isfield(handles,'Template'))
    if(~isempty(handles.Template))
        image=handles.Template;
        handles.Config.Width=size(handles.Template,1);
        handles.Config.Height=size(handles.Template,2);
    else
        image=zeros(round(handles.Config.Width),round(handles.Config.Height));
    end
else
    image=zeros(round(handles.Config.Width),round(handles.Config.Height));
end
handles.Config.image=image;

% Create parameters options
set(handles.PopMenu_Parameterlist,'String',{'Width','Length','Area','Orientation','Grow Speed','Maximum Angle','Curve length','Persistence L <Cos>','Persistence L <R2>'});

% Update template image and data
handles=updateImageAll(handles);
handles=updatedynamicData(handles);

function Buttom_PostAnalysisDataDistribution_Loaddata_Callback(hObject, eventdata, handles)
Value=get(handles.List_Objects,'Value');
if(Value>0)
    try
         handles=InitLoad(handles,pathname);
    catch
        disp('Cannot initialize data in the data path.')
    end
end
handles=InitData(handles);
guidata(hObject, handles);
function handles=ButtonsControl(handles,Option)
set(handles.List_Objects,'Enable',Option);
set(handles.button_objectRemove,'Enable',Option);
set(handles.PopMenu_Parameterlist,'Enable',Option);

function handles=updateImageAll(handles)
% Update image
if(ndims(handles.Config.image)<3)
    image3d=repmat(handles.Config.image,[1 1 3]);
else
    image3d=handles.Config.image(:,:,1:3);
end
demo_image3all=zeros(size(image3d));
for i=1:handles.Config.NumOfData
    demo_image=zeros(size(handles.Config.image));
    color=rand(1,3);
    matdata=handles.Data{i}.PixelIDs;
    [idx,idy]=ind2sub([handles.Config.Width handles.Config.Height],matdata);
    for j=1:length(idx)
        demo_image(idx(j),idy(j))=1;
    end
    demo_image=bwmorph(demo_image,'dilate');
    demo_image3=cat(3, color(1).*demo_image, color(2).*demo_image,color(3).*demo_image);
    demo_image3all=demo_image3all+demo_image3;
end
image3d=imfuse(demo_image3all,image3d,'blend','Scaling','joint');
imshow(image3d,'Parent',handles.Image_Axes)
function handles=updateImageOne(handles)
% Update image
if(ndims(handles.Config.image)<3)
    image3d=repmat(handles.Config.image,[1 1 3]);
else
    image3d=handles.Config.image(:,:,1:3);
end
demo_image3all=zeros(size(image3d));
demo_image=zeros(size(handles.Config.image));
color=[0 1 0];
matdata=handles.Data{handles.Config.CurrentID}.PixelIDs;
[idx,idy]=ind2sub([handles.Config.Width handles.Config.Height],matdata);
for j=1:length(idx)
    demo_image(idx(j),idy(j))=1;
end
demo_image=bwmorph(demo_image,'dilate');
demo_image3=cat(3, color(1).*demo_image, color(2).*demo_image,color(3).*demo_image);
demo_image3all=demo_image3all+demo_image3;
image3d=imfuse(demo_image3all,image3d,'blend','Scaling','joint');
imshow(image3d,'Parent',handles.Image_Axes)

function handles=updateobjectsdata(handles)
ResultsAll=zeros(handles.Config.NumOfData,11);
ResultName={};
for i_check=1:handles.Config.NumOfData
    i_check
    if(~isempty(handles.Data{i_check}.ResultsData))
        ResultsAll(i_check,:)=handles.Data{i_check}.ResultsData(end,:);
        ResultName{i_check}=num2str(handles.Data{i_check}.ResultsData(end,1));
    else
        ResultName{i_check}='Error';
    end
end
set(handles.Text_NumofDetection,'String',num2str(handles.Config.NumOfData));
set(handles.List_Objects,'String',ResultName);
set(handles.ConversionFactor,'String',num2str(handles.Config.PixelSpacing*1e9,'%05.4f'));
set(handles.Time_interval,'String',num2str(handles.Config.TimeInterval,'%05.4f'));
set(handles.FilterSize,'String',num2str(handles.Config.FilterSize));
% Cauclate averaged values
AveragedValue=mean(ResultsAll(:,3:end),1);
STdevValue=std(ResultsAll(:,3:end),[],1);
% Update table
set(handles.Averaged_Width,'String',num2str(AveragedValue(1)*1e6,'%05.5f'));
set(handles.Averaged_Length,'String',num2str(AveragedValue(2)*1e6,'%05.5f'));
set(handles.Averaged_Area,'String',num2str(AveragedValue(3)*1e6*1e6,'%05.5f'));
set(handles.Averaged_Orientation,'String',num2str(AveragedValue(4),'%05.5f'));
set(handles.Averaged_Growspeed,'String',num2str(AveragedValue(5)*1e6*1e6,'%05.5f'));
set(handles.Averaged_MaximumAngle,'String',num2str(AveragedValue(6),'%05.5f'));
set(handles.Averaged_Curvelength,'String',num2str(AveragedValue(7)*1e6,'%05.5f'));
set(handles.Averaged_PLCos,'String',num2str(AveragedValue(8),'%05.5f'));
set(handles.Averaged_PLR2,'String',num2str(AveragedValue(9),'%05.5f'));

set(handles.Averaged_Width_st,'String',num2str(STdevValue(1)*1e6,'%05.5f'));
set(handles.Averaged_Length_st,'String',num2str(STdevValue(2)*1e6,'%05.5f'));
set(handles.Averaged_Area_st,'String',num2str(STdevValue(3)*1e6*1e6,'%05.5f'));
set(handles.Averaged_Orientation_st,'String',num2str(STdevValue(4),'%05.5f'));
set(handles.Averaged_Growspeed_st,'String',num2str(STdevValue(5)*1e6*1e6,'%05.5f'));
set(handles.Averaged_MaximumAngle_st,'String',num2str(STdevValue(6),'%05.5f'));
set(handles.Averaged_Curvelength_st,'String',num2str(STdevValue(7)*1e6,'%05.5f'));
set(handles.Averaged_PLCos_st,'String',num2str(STdevValue(8),'%05.5f'));
set(handles.Averaged_PLR2_st,'String',num2str(STdevValue(9),'%05.5f'));
function handles=updatePopMenu(handles,OptionName)
CurrentData=handles.Data{handles.Config.CurrentID}.ResultsData;
xdata=CurrentData(:,2)*handles.Config.TimeInterval;
i_option=1;
switch OptionName
    case 'Width' 
        i_option=3;
        ydata=CurrentData(:,i_option)*1e6;
        plot(handles.Axes_PostAnalysisDataDistribution,xdata,ydata,'ok');
        ylabel(handles.Axes_PostAnalysisDataDistribution,'Size (\mum)');
    case 'Length'
        i_option=4;
        ydata=CurrentData(:,i_option)*1e6;
        plot(handles.Axes_PostAnalysisDataDistribution,xdata,ydata,'ok');
        ylabel(handles.Axes_PostAnalysisDataDistribution,'Size (\mum)');
    case 'Area'
        i_option=5;
        ydata=CurrentData(:,i_option)*1e6*1e6;
        plot(handles.Axes_PostAnalysisDataDistribution,xdata,ydata,'ok');
        ylabel(handles.Axes_PostAnalysisDataDistribution,'Area (\mum^2)');
    case 'Orientation'
        i_option=6;
        ydata=CurrentData(:,i_option);
        plot(handles.Axes_PostAnalysisDataDistribution,xdata,ydata,'ok');
        ylabel(handles.Axes_PostAnalysisDataDistribution,'Angle (degrees)');
    case 'Grow Speed'
        i_option=7;
        ydata=CurrentData(:,i_option)*1e6*1e6;
        plot(handles.Axes_PostAnalysisDataDistribution,xdata,ydata,'ok');
        ylabel(handles.Axes_PostAnalysisDataDistribution,'Grow Speed (\mum^2/s)');
    case 'Maximum Angle'
        i_option=8;
        ydata=CurrentData(:,i_option);
        plot(handles.Axes_PostAnalysisDataDistribution,xdata,ydata,'ok');
        ylabel(handles.Axes_PostAnalysisDataDistribution,'Angle (degrees)');
    case 'Curve length'
         i_option=9;
         ydata=CurrentData(:,i_option)*1e6;
         plot(handles.Axes_PostAnalysisDataDistribution,xdata,ydata,'ok');
         ylabel(handles.Axes_PostAnalysisDataDistribution,'Size (\mum)');
    case 'Persistence L <Cos>'
        i_option=10;
         ydata=CurrentData(:,i_option);
         plot(handles.Axes_PostAnalysisDataDistribution,xdata,ydata,'ok');
         ylabel(handles.Axes_PostAnalysisDataDistribution,'Size (\mum)');
    case 'Persistence L <R2>'
         i_option=11;
         ydata=CurrentData(:,i_option);
         plot(handles.Axes_PostAnalysisDataDistribution,xdata,ydata,'ok');
         ylabel(handles.Axes_PostAnalysisDataDistribution,'Size (\mum)');

end
hold(handles.Axes_PostAnalysisDataDistribution,'on');
if(length(xdata)>handles.Config.FilterSize && i_option~=1)
        yfiltered = medfilt1(ydata,handles.Config.FilterSize,'zeropad');
        plot(xdata,yfiltered,'--b','Parent',handles.Axes_PostAnalysisDataDistribution);hold(handles.Axes_PostAnalysisDataDistribution,'off');
        legend(handles.Axes_PostAnalysisDataDistribution,'Original samples','Median filter');
        miny=min(yfiltered(:));
        maxy=max(yfiltered(:));
        if(miny~=maxy)
            ylim(handles.Axes_PostAnalysisDataDistribution,[miny,maxy]);
        end
else
        legend(handles.Axes_PostAnalysisDataDistribution,'Original samples');
        hold(handles.Axes_PostAnalysisDataDistribution,'off');
end
xlabel(handles.Axes_PostAnalysisDataDistribution,'Time (s)');  
function handles=updatedynamicData(handles)
% update object dynamic data
set(handles.objectID,'String',num2str(handles.Config.CurrentID));
CurrentData=handles.Data{handles.Config.CurrentID}.ResultsData;
if(~isempty(CurrentData))
    for j=1:size(CurrentData,1)
        Time{j}=num2str(CurrentData(j,2)*handles.Config.TimeInterval,'%05.5f');
    end
    % Cauclate averaged values
    Averaged_objectValue=mean(CurrentData(:,3:end),1);
    Stdev_objectValue=std(CurrentData(:,3:end),[],1);
    % Update table
    set(handles.object_Width,'String',num2str(Averaged_objectValue(1)*1e6,'%05.5f'));
    set(handles.object_Length,'String',num2str(Averaged_objectValue(2)*1e6,'%05.5f'));
    set(handles.object_Area,'String',num2str(Averaged_objectValue(3)*1e6*1e6,'%05.5f'));
    set(handles.object_Orientation,'String',num2str(Averaged_objectValue(4),'%05.5f'));
    set(handles.object_Growspeed,'String',num2str(Averaged_objectValue(5)*1e6*1e6,'%05.5f'));
    set(handles.object_MaximumAngle,'String',num2str(Averaged_objectValue(6),'%05.5f'));
    set(handles.object_Curvelength,'String',num2str(Averaged_objectValue(7)*1e6,'%05.5f'));
    set(handles.object_PLCos,'String',num2str(Averaged_objectValue(8),'%05.5f'));
    set(handles.object_PLR2,'String',num2str(Averaged_objectValue(9),'%05.5f'));
    
    set(handles.object_Width_st,'String',num2str(Stdev_objectValue(1)*1e6,'%05.5f'));
    set(handles.object_Length_st,'String',num2str(Stdev_objectValue(2)*1e6,'%05.5f'));
    set(handles.object_Area_st,'String',num2str(Stdev_objectValue(3)*1e6*1e6,'%05.5f'));
    set(handles.object_Orientation_st,'String',num2str(Stdev_objectValue(4),'%05.5f'));
    set(handles.object_Growspeed_st,'String',num2str(Stdev_objectValue(5)*1e6*1e6,'%05.5f'));
    set(handles.object_MaximumAngle_st,'String',num2str(Stdev_objectValue(6),'%05.5f'));
    set(handles.object_Curvelength_st,'String',num2str(Stdev_objectValue(7)*1e6,'%05.5f'));
    set(handles.object_PLCos_st,'String',num2str(Stdev_objectValue(8),'%05.5f'));
    set(handles.object_PLR2_st,'String',num2str(Stdev_objectValue(9),'%05.5f'));
    % Update list
    set(handles.List_Frames,'String',Time);
    % Update plot
    Options=cellstr(get(handles.PopMenu_Parameterlist,'String'));
    OptionName=Options{get(handles.PopMenu_Parameterlist,'Value')};
    handles=updatePopMenu(handles,OptionName);
end

function PopMenu_Parameterlist_Callback(hObject, eventdata, handles)
Options=cellstr(get(hObject,'String'));
OptionName=Options{get(hObject,'Value')};
handles=updatePopMenu(handles,OptionName);
handles=updateImageOne(handles);
handles=updatedynamicData(handles);
guidata(hObject, handles);
function PopMenu_Parameterlist_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function List_Frames_Callback(hObject, eventdata, handles)
function List_Frames_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function List_Objects_Callback(hObject, eventdata, handles)
handles.Config.CurrentID=get(hObject,'Value');
handles=updateImageOne(handles);
handles=updatedynamicData(handles);
guidata(hObject, handles);
function List_Objects_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function button_objectRemove_Callback(hObject, eventdata, handles)
Value=get(handles.List_Objects,'Value');
if(Value>0)
    try
        handles.Config.CurrentID=Value;
        choice = questdlg('Delete the file in the folder?', ...
        'Detection ', ...
        'Yes','No','No');
        switch choice
            case 'Yes'
                delete(handles.Data{Value}.Path);
            case 'No'
        end
        handles.Data(Value)=[];
        handles.Config.NumOfData=length(handles.Data);
        if(handles.Config.NumOfData<Value)
            handles.Config.CurrentID=handles.Config.NumOfData;
            set(handles.List_Objects,'Value',handles.Config.NumOfData);
        else
            set(handles.List_Objects,'Value',Value);
        end
    catch
        
    end
end
if(handles.Config.NumOfData>0)
    handles=updateobjectsdata(handles);
    handles=updateImageAll(handles);
    handles=updatedynamicData(handles);
else
    
end
guidata(hObject, handles);

% GUI parameter inputs
function Time_interval_Callback(hObject, eventdata, handles)
Numstring=get(hObject,'String');
if(isempty(Numstring))
    set(hObject,'String',num2str(handles.Config.TimeInterval));
    guidata(hObject, handles);
    return
end
[Value, status] = str2num(Numstring);
if(status==0)
    set(hObject,'String',num2str(handles.Config.TimeInterval));
    guidata(hObject, handles);
    return
end
% Update Parameters (Configuration & instance)
if(Value>0)
    handles.Config.TimeInterval=Value;
end
set(hObject,'String',num2str(handles.Config.TimeInterval,'%10.4f'));
guidata(hObject, handles);
function Time_interval_CreateFcn(hObject, eventdata, handles)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function FilterSize_Callback(hObject, eventdata, handles)
Numstring=get(hObject,'String');
if(isempty(Numstring))
    set(hObject,'String',num2str(handles.Config.FilterSize,'%10.0f'));
    guidata(hObject, handles);
    return
end
[Value, status] = str2num(Numstring);
if(status==0)
    set(hObject,'String',num2str(handles.Config.FilterSize,'%10.0f'));
    guidata(hObject, handles);
    return
end
% Update Parameters (Configuration & instance)
handles.Config.FilterSize=Value;
handles=updateImageOne(handles);
handles=updatedynamicData(handles);
set(hObject,'String',num2str(handles.Config.FilterSize,'%10.0f'));
guidata(hObject, handles);
function FilterSize_CreateFcn(hObject, eventdata, handles)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function ConversionFactor_Callback(hObject, eventdata, handles)
Numstring=get(hObject,'String');
if(isempty(Numstring))
    set(hObject,'String',num2str(handles.Config.PixelSpacing,'%05.4f'));
    guidata(hObject, handles);
    return
end
[Value, status] = str2num(Numstring);
if(status==0)
    set(hObject,'String',num2str(handles.Config.PixelSpacing,'%05.4f'));
    guidata(hObject, handles);
    return
end
% Update Parameters (Configuration & instance)
if(Value>0)
    handles.Config.PixelSpacing=Value;
end
set(hObject,'String',num2str(handles.Config.PixelSpacing,'%05.4f'));
guidata(hObject, handles);
function ConversionFactor_CreateFcn(hObject, eventdata, handles)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function pushbutton_RemoveTime_Callback(hObject, eventdata, handles)
Value=get(handles.List_Frames,'Value');
if(Value>0)
    try
        handles.Data{handles.Config.CurrentID}.ResultsData(Value,:)=[];
    catch
        
    end
end
handles=updatedynamicData(handles);
guidata(hObject, handles);


% --------------------------------------------------------------------
function LoadNewData_Callback(hObject, eventdata, handles)
pathname = uigetdir('','Select a path');
if  isempty(pathname)
    disp('Cancled.');
    return;
else
    disp(['User selected ', pathname]);
end
handles=InitLoad(handles,pathname);
handles=InitData(handles);
guidata(hObject, handles);% 

% --------------------------------------------------------------------
function ExportXML_Callback(hObject, eventdata, handles)
handles.Config.NumOfData=length(handles.Data);
handles.Config.ResultsName={'Object Id' 'Time (seconds)' 'Width (micro-meter)' 'Length (micrometer)' 'Area (micrometer^2)' 'Oerientation (degrees)' 'Growing Speed (micrometer^2/second)' 'Maximum Angle' 'Backbone length (micrometer)' 'Persistence length <Cos> (micrometer)' 'Persistence length <R2> (micrometer)'};
                        
UnitVector=[1  handles.Config.TimeInterval 1e6 1e6 1e6*1e6  1 1e6*1e6/handles.Config.TimeInterval 1 1e6 1 1];
if(handles.Config.NumOfData>0)
    try
        f = waitbar(0,'Please wait...');
        for i=1:handles.Config.NumOfData
            waitbar(double(i/handles.Config.NumOfData),f,'Please wait...');
            [Path,Name,Ext]=fileparts(handles.Data{i}.Path);
            FileName=strcat(Path,'\',Name,'.xlsx');
            
            handles.Config.PixelSpacing=handles.Data{i}.Config.System.PixelSpacing;

            Data=handles.Data{i}.ResultsData;
            if(size(Data,1)>0)
                Data=Data.*repmat(UnitVector,[size(Data,1) 1]);
            else
                Data=[];
            end
            
            xlswrite(FileName,handles.Config.ResultsName,'Results','A1');
            xlswrite(FileName,Data,'Results','A2');
            
            matdata=handles.Data{i}.PixelIDs;
            if(~isempty(matdata))
                Corrodinates=zeros(length(matdata),2);
                [Corrodinates(:,1),Corrodinates(:,2)]=ind2sub([handles.Config.Width handles.Config.Height],matdata);
            else
                Corrodinates=[];
            end
            xlswrite(FileName,{'Template size:',...
                    num2str(handles.Data{i}.Config.Height),...
                    ' x ',...
                    num2str(handles.Data{i}.Config.Width),...
                    ' Resize factor:',...
                    num2str(handles.Data{i}.Config.Resize),...
                    ' Offset (pixels in the original image) :',...
                    num2str(handles.Data{i}.Config.Offset(1)),...
                    ', ',...
                    num2str(handles.Data{i}.Config.Offset(2)),...
                    },...
                    'Pixels','A1');
            xlswrite(FileName,Corrodinates,'Pixels','A2');
        end
        close(f);

    catch
        if(~isempty(f))
            close(f);
        end
        error('Data format error.')
    end
else
    disp('No data files available.');
    return
end
