%   This file is a part of UMUTracker Project.The UMUTracker is currently under active development.
%	Related information can be found in the paper :
%
%	Hanqing Zhang, Tim Stangner, Krister Wiklund, Alvaro Rodriguez, Magnus Andersson
%	UmUTracker: A versatile MATLAB program for automated particle tracking of 2D light microscopy or 3D digital holography data
%
%	We welcome comments and contributions to the documentation and code of UMUTracker to help us improve the implementation.
%
%   Version: 1.Initial version:  Hanqing Zhang,hanqing.zhang@umu.se
%
classdef dhm_reconstruction < INTERFACE_dhm_reconstruction
    properties
       SystemConfig=[]; 
    end
    methods
        function obj=dhm_reconstruction(Config)
            if(nargin>0)
                try
                    obj.ReconstructConfig=Config.ReconstructConfig;
                catch
                    error('CLASS_DHMTracking_IMPLEMENTATION: Parameters Error');
                end
            else
                obj.SystemConfig.Extern='.avi';
                % Parameters 
                obj.SystemConfig.GausLayers=5;
                obj.SystemConfig.StdevMultiple=10; % standard deviation * StdevMultiple
                obj.SystemConfig.TimeInterval=10; % 
                % Outliers
                obj.SystemConfig.OutlierValue=0.1;
                % AutoFocus
                obj.SystemConfig.AutoFocusActive=1; %
                obj.SystemConfig.SelectedFrame=1; %
                % ROI
                obj.SystemConfig.Template=[];
                obj.SystemConfig.TemplateROI=[];
                obj.SystemConfig.TemplateRaius=200;
                % Drift
                obj.SystemConfig.DriftCorrectionActive=1;
                obj.SystemConfig.DriftCalibrationDone=0;
                obj.SystemConfig.DriftCalibration=[];
                obj.SystemConfig.Results=[];
                % Background Norm
                obj.SystemConfig.BackgroundNormActive=1;
                %% Parallel computing
                obj.ReconstructConfig.GPUActive=1;
                %% Reconstruct
                obj.ReconstructConfig.PixelSpacing=0.160e-6; % CCD size/pixel num/Magnification
            end
        end
    end
    %% Implementation
    methods      
        function obj=SetConfig(obj,Field,Value)
            switch Field
                case 'TimeInterval'
                    obj.ReconstructConfig.TimeInterval=Value;
                case 'PixelSpacing'
                    obj.ReconstructConfig.PixelSpacing=Value;
                case 'GausLayers'
                    obj.SystemConfig.GausLayers=Value;
            end
         
        end             
        function Imageout=Preprocess(obj,Image,Mode)
            if(nargin<=2)
                Mode=1;
            end
            if(ndims(Image)==3)
                Imageout=rgb2gray(Image);
            else
                Imageout=Image;
            end
            % Convert to double 0-1
            if(isa(Imageout,'uint8'))
                Imageout=double(Imageout)/255;
            elseif(isa(image(1),'uint16'))
                Imageout=double(Imageout)/65535;
            else
                Imageout=double(Imageout);
            end
            if(~isempty(obj.SystemConfig.TemplateROI) && Mode==1)
                if(mod(obj.SystemConfig.TemplateROI(3),2)~=0)
                    obj.SystemConfig.TemplateROI(3)=obj.SystemConfig.TemplateROI(3)-1;
                end
                if(mod(obj.SystemConfig.TemplateROI(4),2)~=0)
                    obj.SystemConfig.TemplateROI(4)=obj.SystemConfig.TemplateROI(4)-1;
                end
                Imageout=imcrop(Imageout,obj.SystemConfig.TemplateROI);
            end
            obj.SystemConfig.Template=Imageout;
        end
        function IntensityResult=ImageTranslation_Intensity(~,SourcePattern,TargetPattern,layers)
            % ؽͼ߽չ
            SourceMedian=median(SourcePattern(:));
            TargetMedian=median(TargetPattern(:));
            [HeightImage,WidthImage,~]=size(SourcePattern);
            % 
            layers=max(layers,1);
            SourceIntensity=double(SourcePattern)-SourceMedian;
            TargetIntensity=double(TargetPattern)-TargetMedian; 
            ObjectReconstructIntensity=zeros(HeightImage+size(TargetIntensity,1)-1,WidthImage+size(TargetIntensity,2)-1,layers);
            for check_i=1:1:layers
                SourceGaussianFitlered = imgaussfilt(SourceIntensity,0.8+0.8*(check_i-1));
                ObjectReconstructIntensity(:,:,check_i) = xcorr2(SourceGaussianFitlered,TargetIntensity);
            end
            IntensityResult=findExtremValues3D(ObjectReconstructIntensity,1,2,0);
            IntensityResult.locx=IntensityResult.locx-floor(HeightImage/2)-1;
            IntensityResult.locy=IntensityResult.locy-floor(WidthImage/2)-1;
        end
    end
    
    
end